/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
*
** NAME
**   setupapi.h
**
** DESCRIPTION
**   Various Common functions shared by NT and Unix.
**
** NOTES
**
** HISTORY
** $Log: setupapi.h,v $
** Revision 1.1.1.1  2005/07/29 22:16:28  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:26  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/07/27 01:04:49  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.2  2005/03/01 01:45:12  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.8  2001/11/06 01:51:28  ahakim
** XXX AOL/NSCP Copyright XXX
**
** Revision 1.1.1.7  1999/11/10 23:47:50  cxue
** setupFileUpdateLines
**
** Revision 1.1.1.6  1999/11/08 18:49:16  cxue
** setupSetDefaultLdapInfo
**
** Revision 1.1.1.5  1999/10/20 20:45:53  cxue
** new api to write name/value pair to configuration file like magnus.conf
**
** Revision 1.1.1.4  1999/09/23 00:48:08  cxue
** Enable setupGetDiskFreeSpace for unix
**
** Revision 1.1.1.3  1999/05/18 17:46:31  cxue
** setupAddNameToList and setupFindNameInList
**
** Revision 1.1.1.2  1999/03/13 02:43:51  cxue
** setupGetDiskFreeSpace for NT
**
** Revision 1.1.1.1  1999/03/04 23:26:56  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.15  1999/01/16 01:32:12  gbeasley
** *** empty log message ***
**
** Revision 1.1.2.14  1999/01/14 00:40:10  cxue
** setupDeleteDir
**
** Revision 1.1.2.13  1999/01/11 00:11:18  gbeasley
** more api
**
** Revision 1.1.2.12  1998/12/11 01:08:47  gbeasley
** mem fuctions
**
** Revision 1.1.2.11  1998/11/24 03:25:16  adam
** SHA1_Hash and SHA1_HashBuf already defined in NSS; don't make them external symbols.
**
** Revision 1.1.2.10  1998/11/17 23:14:12  cxue
** move uint32 typedef to global.h
**
** Revision 1.1.2.9  1998/11/17 00:22:35  cxue
** uint32 conflicts prtypes.h
**
** Revision 1.1.2.8  1998/11/14 01:14:30  gbeasley
** unified api
**
** Revision 1.1.2.7  1998/11/10 19:05:48  cxue
** setupExecProgram
**
** Revision 1.1.2.6  1998/11/09 17:37:08  cxue
** move InstallLog from ux-util.h
**
** Revision 1.1.2.5  1998/11/04 21:49:03  cxue
** comment
**
** Revision 1.1.2.4  1998/10/31 00:19:45  ahakim
** 330089: Win95 Setup Work
** - remove admin check for 95 (also NT) for setup and uninst
** - add setupIsWindowsNT() function
**
** Revision 1.1.2.3  1998/10/28 17:50:42  cxue
** new apis
**
** Revision 1.1.2.2  1998/10/14 00:53:25  gbeasley
** inf api
**
** Revision 1.1.2.1  1998/10/13 21:04:53  gbeasley
** changing nss to setup
**
**
**/

#ifndef SETUPAPI_H
#define SETUPAPI_H

#include "global.h"

#ifdef __cplusplus
extern "C" {
#endif


/*
* Conversion API's

*/
NSAPI_EXTERN char *setupSHA1_pw_enc( char *pwd );
NSAPI_EXTERN int setupSHA1_pw_cmp( char *userpwd, char *dbpwd );
NSAPI_EXTERN int setupBase64_decode( char *src, unsigned char *dst );
NSAPI_EXTERN int setupBase64_Encode( unsigned char *src, char *dst, int srclen, int lenused );
NSAPI_EXTERN char *setupUTF8ToLocal(const char *utf8);
NSAPI_EXTERN char *setupLocalToUtf8(const char *src);
NSAPI_EXTERN int  setupIsAsciiBuffer(const char *buf);


/* Memory allocation and free */
NSAPI_EXTERN void *         setupMalloc(unsigned int size);
NSAPI_EXTERN char *         setupStrdup(char *src);
NSAPI_EXTERN void           setupFree(char *src);

/* Directory server access */
NSAPI_EXTERN void           setupGetDefaultLdapInfo (const char *sroot,char **ldapURL,char **ldapUser,char **adminDomain);
NSAPI_EXTERN void           setupSetDefaultLdapInfo(const char *sroot, char * ldapURL, char * ldapUser, char *adminDomain);
NSAPI_EXTERN char           *setupFormConfigLdapURL(const char *ldapURL);
NSAPI_EXTERN char           *setupStripConfigLdapURL(const char *configURL);
NSAPI_EXTERN int            setupIsValidLdapServer(const char* pszHost, int nPort, const char* pszSuffix,
                                                 const char* pszUserDN, const char* pszPassword);
NSAPI_EXTERN int            setupIsValidLdapUser(const char* pszHost, int nPort, const char* pszSuffix,
                                               char** pszUser, const char* pszPassword, int bReturnDN);
NSAPI_EXTERN int            setupParseLdapUrl(char * szldapURL, char ** szldapDomain, unsigned int *uldapPort, char ** szldapDN);
NSAPI_EXTERN char           *setupFormAdminDomainDN(const char *domain);
NSAPI_EXTERN char           *setupStripAdminDomainDN(const char *domainDN);
NSAPI_EXTERN int            setupInsertPluginLdifEntries(const char *ldapURL, const char *adminDomain,
                                                         const char *ldapUser, const char *ldapPwd, const char *ldifFile);
NSAPI_EXTERN int            setupInsertPluginSchemaEntries(const char *ldapURL, const char *ldapUser, const char *ldapPwd, const char *schemaFile);

/* Logging, and Inf file access  */
NSAPI_EXTERN int            setupSetLogFile(const char *pszFile);
NSAPI_EXTERN int            setupLogMessage(const char *level, const char *who, const char *msg, ...);

NSAPI_EXTERN int            setupLog(const char *pszFile, const char *pszText);


NSAPI_EXTERN char           *setupGetFileString(const char* pszKey, const char* sep, const char* pszFileName);
NSAPI_EXTERN int            setupGetFileInt(const char* pszKey, const char* sep, const char* pszFileName);
NSAPI_EXTERN char           *setupGetInfString(const char* pszSection, const char* pszKey, const char* pszdefault, const char* pszFileName);
NSAPI_EXTERN int            setupGetInfInt(const char* pszSection, const char* pszKey, int iDefault, const char* pszFileName);
NSAPI_EXTERN int            setupWriteInfString(const char* pszSection, const char* pszKey, const char* pszString, const char* pszFileName);
NSAPI_EXTERN int            setupWriteInfInt(const char* pszSection, const char* pszKey, int i, const char* pszFileName);


/*********************************************************************
**
** FUNCTION:    setupFindNameInList
** DESCRIPTION: Check if given name is already on the list.  
** INPUTS:      listValue:   name/value list
**              name:        name to look for
** RETURN:      True:        name is in the list
**              False:       name is not in the list
**********************************************************************
*/
NSAPI_EXTERN Bool           setupFindNameInList(const char *listValue, const char *name);

/*********************************************************************
**
** FUNCTION:    setupAddNameToList
** DESCRIPTION: 
**    Add name to a list.  If list is not there, it will create
**    new one. If the name is already on the list, it will return
**    a copy of the list. 
** INPUTS:      listValue:   name/value list
**              name:        name to add to the list
** RETURN:      new name/value list
**********************************************************************
*/
NSAPI_EXTERN char           * setupAddNameToList(const char *listValue, const char *name);


/* Local file system  */
NSAPI_EXTERN int            setupIsDirEmpty(const char * pszPath);
NSAPI_EXTERN int            setupFileExists(const char * pszFileName);
NSAPI_EXTERN int            setupDirExists(const char * pszDirName);
NSAPI_EXTERN int            setupCreateDir(const char * pszDirName, int mode);
NSAPI_EXTERN int            setupDeleteDir(const char *path);

NSAPI_EXTERN int            setupConvertPathSlashes(const char *  lpszPath, char *  lpszNewPath, int bType );
NSAPI_EXTERN int            setupGetNextFileInDirectory ( long hFile, const char * path, char * lpFileName, int lpFileNameLen );
NSAPI_EXTERN int            setupGetNextSubDirectory( long hFile, char *path, char *lpSubDirectoryName, int lpSubDirectoryNameLen, char *lpSubDirectoryPrefix );
NSAPI_EXTERN int            setupDeleteRecursively (const char * path );

/* Network */
NSAPI_EXTERN char           *setupGetHostName(void);
NSAPI_EXTERN char           *setupGetNetworkDomain(void);
NSAPI_EXTERN unsigned int   setupGetRandomNetworkPort(void);
NSAPI_EXTERN int            setupIsValidNetworkPort(unsigned int nPort);

/* Platform */
NSAPI_EXTERN int            setupIsWindowsNT(void);

NSAPI_EXTERN unsigned int   setupExecProgram(const char *program, const char *where);
NSAPI_EXTERN void           setupGetGmtime(char *buf, int buflen);

NSAPI_EXTERN unsigned long  setupGetDiskFreeSpace(const char *path);
NSAPI_EXTERN int            setupFileUpdateLines(char *oldLines[], char *newLines[], char *filename);


#ifdef __cplusplus
}
#endif

/*
 ************************************************************************
 * Install Log Error Levels
 ***********************************************************************
 */

extern const char *SETUP_LOG_FATAL;
extern const char *SETUP_LOG_INFO;
extern const char *SETUP_LOG_WARN;
extern const char *SETUP_LOG_START;
extern const char *SETUP_LOG_SUCCESS;

#ifdef __cplusplus
/*
** CLASS:
**   InstallLog
**
** DESCRIPTION:
**   The Install Log class
**
** NOTE:
**
*/


class InstallLog
{
public:
   InstallLog (const char *fileName);

   ~InstallLog ();

   Bool isValid() const;

   void logTime(const char *who);
   void logMessage (const char *level, const char *who, const char *msg, ...);
   static void log (const char *fileName, const char *msg);

   static char *_logFile;

private:
   Bool _valid;
};

#endif



#endif
