/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/**********************************************************************
**                                                                          **
**  Name: SETUPGLOBAL.H                                                     **
**  Header file for Netscape common Windows NT and UNIX installation defines**
**  Platforms:  UNIX and WIN32                                              **
**  Revision History:                                                       **
**  $Log: setupdefs.h,v $
**  Revision 1.2  2005/10/26 16:38:10  rmeggins
**  Bug(s) fixed: 171793
**  Bug Description: Change setuputil version to 1.0
**  Reviewed by: Noriko (Thanks!)
**  Fix Description: Mostly just typical rebranding stuff.  I found a couple
**  of places in installer.cc that were not listed in the branding.dat so I
**  added them.  I also got rid of the webserv version and the enterprise
**  stuff, which is apparently not used anywhere.
**  Platforms tested: RHEL4
**  Flag Day: no
**  Doc impact: no
**  QA impact: should be covered by regular nightly and manual testing
**  New Tests integrated into TET: none
**
**  Revision 1.1.1.1  2005/07/29 22:16:28  foxworth
**  Importing new setup sdk for open source project
**
**  Revision 1.7  2005/07/29 19:43:26  nkinder
**  164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
**  Revision 1.6  2005/07/27 03:01:38  nhosoi
**  [161408] Open Sourcing Setup Util
**  Replacing "#include <iostream>" with "#include <iostream.h>" for HP-UX.
**
**  Revision 1.5  2005/07/27 01:04:49  nhosoi
**  [161408] Open Sourcing Setup Util
**  1) Setup SDK -> Setup Util
**  2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
**  3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
**  4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
**  5) code cleanup
**
**  Revision 1.4  2005/07/05 22:05:46  nhosoi
**  [161408] Open Sourcing Setup SDK; an attachment (id=116383)
**  adjustment for the gar build
**
**  Revision 1.3  2005/03/01 01:45:12  nhosoi
**  Brand the Setup SDK as "Fedora"; updated copyrights
**
**  Revision 1.2  2005/02/03 18:24:07  nhosoi
**  rebranded setupsdk as brandx; updated the version number to 7.0
**
**  Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
**  Move SetupSDK from 6.2 development branch to TRUNK
**
**  Revision 1.1.1.13.4.2  2003/08/28 18:13:20  djani
**  Bug 601226: Now, the setup allows the programmer to run PrePreInstall programs on Unix platforms. The field PrePreInstall holds the location of the program relative to the setup program that will be executed before the dialogues appear. This is a new feature in the setupsdk.
**
**  Revision 1.1.1.13.4.1  2002/04/12 23:44:21  djani
**  Bug 600161: Backport from SetupsdkBranch to setupsdk_dev branch.
**
**  Revision 1.1.1.13  2001/11/06 01:51:28  ahakim
**  XXX AOL/NSCP Copyright XXX
**
**  Revision 1.1.1.12  2001/09/20 21:30:12  miodrag
**  Change iPlanet 4.5 sdk to Netscape 6.0 sdk
**
**  Revision 1.1.1.11  2000/08/11 01:06:34  peterz
**  change iPlanet/Server4 to iPlanet/Servers
**
**  Revision 1.1.1.10  2000/03/06 01:12:41  peterz
**  change "Netscape" to "iPlanet"
**
**  Revision 1.1.1.9  2000/01/14 04:57:53  peterz
**  add setup_ing_flag SkipProductPage
**
**  Revision 1.1.1.8  1999/12/03 02:55:22  cxue
**  SecurityCheck
**
**  Revision 1.1.1.7  1999/11/13 00:30:35  cxue
**  default version to 4.5
**
**  Revision 1.1.1.6  1999/10/26 23:32:09  cxue
**  FinishMessage to customize the last message
**
**  Revision 1.1.1.5  1999/10/08 21:01:01  cxue
**  RequireDomain to decide whether to do domainname checking
**
**  Revision 1.1.1.4  1999/06/09 17:44:13  cxue
**  remove spaces before #define, stupid AIX compiler
**
**  Revision 1.1.1.3  1999/05/04 20:35:08  cxue
**  change c++ comment to c
**
**  Revision 1.1.1.2  1999/04/02 18:10:23  cxue
**  Move Unix string definition here
**
**  Revision 1.1.1.1  1999/03/04 23:26:57  svbld
**  importing setupsdk41 closer to the trunk
**
**  Revision 1.1.2.14  1999/01/24 00:25:53  gbeasley
**  captilize Netscape/Server4
**
**  Revision 1.1.2.13  1999/01/20 07:47:04  gbeasley
**  340164 installed components have greater version than installable
**
**  Revision 1.1.2.12  1999/01/19 19:51:35  cxue
**  Netscape Server Family --> Netscape Server Products
**
**  Revision 1.1.2.11  1999/01/18 23:45:12  gbeasley
**  replace files
**
**  Revision 1.1.2.10  1999/01/14 02:59:23  gbeasley
**  *** empty log message ***
**
**  Revision 1.1.2.9  1999/01/13 19:09:27  gbeasley
**  setup configurability
**
**  Revision 1.1.2.8  1999/01/13 01:24:08  gbeasley
**  configurable setup
**
**  Revision 1.1.2.7  1999/01/12 20:07:36  gbeasley
**  *** empty log message ***
**
**  Revision 1.1.2.6  1999/01/12 19:45:02  gbeasley
**  *** empty log message ***
**
**  Revision 1.1.2.5  1999/01/10 21:50:44  gbeasley
**  339262 4.1 features
**
**  Revision 1.1.2.4  1998/10/29 04:34:34  pvo
**  313576 - change install.log to setup.log
**
**  Revision 1.1.2.3  1998/10/28 19:08:34  pvo
**  310620 - update build number upon re-install
**
**  Revision 1.1.2.2  1998/10/28 17:50:42  cxue
**  new apis
**
**  Revision 1.1.2.1  1998/10/13 21:04:53  gbeasley
**  changing nss to setup
**
**
****************************************************************************/

#ifndef __SETUPGLOBALS_H
#define __SETUPGLOBALS_H
/* Setup install modes  */
#define SETUP_EXPRESSMODE                    0
#define SETUP_TYPICALMODE                    1
#define SETUP_CUSTOMMODE                     2
#define SETUP_SILENTMODE                     3
#define SETUP_PREPMODE                       4
#define SETUP_NORMAL_INSTALL                 5
#define SETUP_PATCH_INSTALL                  6
#define SETUP_UPDATE_INSTALL                 7
/* Setup wizard page defines*/
#define SETUP_WIZERROR                       -1
#define SETUP_WIZCANCEL                      0
#define SETUP_WIZBACK                        1
#define SETUP_WIZNEXT                        2
/* Setup common files */
#define SETUP_DEFAULT_LDAPSWITCH             "shared/config/dbswitch.conf"
#define SETUP_DS_CONF                        "shared/config/ds.conf"
#define SETUP_ADM_CONF                       "admin-serv/config/adm.conf"
#define SETUP_LDAP_CONF                      "shared/config/ldap.conf"
#define SETUP_LDAP_INFO                      "ldap.info"
/* Setup sie define */
#define SETUP_ADMIN_SIE_GUESS                "cn=admin-serv-%s, cn=Fedora Administration Server, cn=Server Group, cn=%s, %s"
#define SETUP_ADMIN_DOMAIN                   "ou=%s, o=NetscapeRoot"
#define SETUP_DEFAULT_ROOT                   "NetscapeRoot"
#define SETUP_DOMAIN_ROOT                    "o=NetscapeRoot"
/* Setup security defines  */
#define SETUP_SECURITY_DOMESTIC              "DOMESTIC"
#define SETUP_SECURITY_EXPORT                "EXPORT"
#define SETUP_SECURITY_NONE                  "NONE"
/* Setup inf defines */
#define SETUP_REFFILE_INF                    "reffiles.inf"
#define SETUP_INSTALL_LOG                    "setup.log"
#define SETUP_UNINST_INF                     "uninstall.inf"
#define SETUP_INSTALL_INF                    "install.inf"
#define SETUP_SETUP_INF                      "setup.inf"
#define SETUP_SETUP_DIR                      "setup"
#define SETUP_UNINST_DIRECTORY               SETUP_SETUP_DIR
#define SETUP_LOGTAG                         ".log"
#define SETUP_INFTAG                         ".inf"
#define SETUP_SHAREDFILES                    "SharedFiles"
#define SETUP_FILE_INSTALLED                 "yes"
#define SETUP_NT_SYSTEM_FILE                 "NTSYS"
/* Setup inf keywords */
#define SETUP_INF_COMPONENTS                 "Components"
#define SETUP_INF_GENERAL_SECTION            "General"
#define SETUP_INF_SECTION                    "General"
#define SETUP_INF_FORCE_EXPRESS              "ForceExpressInstall"
#define SETUP_INF_NAME                       "Name"
#define SETUP_INF_DESC                       "Description"
#define SETUP_INF_ISMCC                      "IsMCC"
#define SETUP_INF_COMPATIBLE                 "Compatible"
#define SETUP_INF_VERSION                    "Version"
#define SETUP_INF_BUILDNUMBER                "BuildNumber"
#define SETUP_INF_SERIALNUMBER               "SerialNumber"
#define SETUP_INF_SCHEMAVERSION              "LdapSchemaVersion"
#define SETUP_INF_REVISION                   "Revision"
#define SETUP_INF_DEPEND                     "Dependencies"
#define SETUP_INF_NICKNAME                   "NickName"
#define SETUP_INF_SECURITY                   "Security"
#define SETUP_INF_APPLYTO                    "ApplyTo"
#define SETUP_INF_SIZE                       "Size"
#define SETUP_INF_VISIBLE                    "Visible"
#define SETUP_INF_MANDATORY                  "Mandatory"
#define SETUP_INF_CHECK                      "Checked"
#define SETUP_INF_ISADMIN                    "IsAdmin"
#define SETUP_INF_ISLDAP                     "IsLDAP"
#define SETUP_INF_USELDAP                    "UseLDAP"
#define SETUP_INF_USELDAP_UG                 "UserDirectoryAuth"
#define SETUP_INF_FILENAME                   "ComponentInfoFile"
#define SETUP_INF_SOURCEPATH                 "SourcePath"
#define SETUP_INF_DLL                        "PlugIn"
#define SETUP_INF_PREPREINSTALL              "PrePreInstall"
#define SETUP_INF_PREINSTALL                 "PreInstall"
#define SETUP_INF_ASKOPTFUNC                 "AskOptions"
#define SETUP_INF_GETSUMFUNC                 "GetSummary"
#define SETUP_INF_WGCACHEFUNC                "WriteGlobalCache"
#define SETUP_INF_WLCACHEFUNC                "WriteLocalCache"
#define SETUP_INF_RGCACHEFUNC                "ReadGlobalCache"
#define SETUP_INF_RLCACHEFUNC                "ReadLocalCache"
#define SETUP_INF_REG_NYR                    "Registration"
#define SETUP_INF_INSTALLFUNC                "Install"
#define SETUP_INF_POSTINSTALL                "PostInstall"
#define SETUP_INF_RCINFFUNC                  "ReadInf"
#define SETUP_INF_PREUNINSTALL               "PreUnInstall"
#define SETUP_INF_POSTUNINSTALL              "PostUnInstall"
#define SETUP_INF_PATCHID                    "PatchID"
#define SETUP_INF_PATCHES                    "Patches"
#define SETUP_INF_TYPE                       "Type"
#define SETUP_INF_SYSTEM32_ARCHIVE           "System32Archive"
#define SETUP_INF_SERVERROOT_ARCHIVE         "Archive"
#define SETUP_INF_ARCHIVE                    "Archive"
#define SETUP_INF_OVERRIDEUNINSTALL          "OverRideUninstall"
#define SETUP_INF_KEEP_DIRS                  "KeepDirectories"
#define SETUP_INF_KEEP_FILES                 "KeepFiles"
#define SETUP_INF_REMOVABLE                  "Removable"
#define SETUP_INF_ALIAS                      "Alias"
#define SETUP_INF_CREATIONDATE               "CreationDate"
#define SETUP_INF_LANGUAGES                  "Languages"
#define SETUP_INF_EXPIRY                     "Expires"
#define SETUP_INF_DEFAULT_LANGUAGE           "DefaultAcceptLanguage"
#define SETUP_INF_DESTINATION                "Destination"
#define SETUP_INF_UPDATE_LDIF                "UpdateLdif"
#define SETUP_INF_UPDATE_SCHEMA              "UpdateSchema"
#define SETUP_INF_REMOVE_LDIF                "RemoveLdif"
#define SETUP_INF_SCHEMA_LDIF                "SchemaLdif"
#define SETUP_INF_RESTORE_FILES              "RestoreFiles"
#define SETUP_INF_BACKUP_FILES               "BackupFiles"
#define SETUP_INF_START_PROGRAM              "StartProgram"
#define SETUP_INF_MODE                       "MODE"
#define SETUP_INF_EXPRESS_MODE               "Express"
#define SETUP_INF_TYPICAL_MODE               "Typical"
#define SETUP_INF_CUSTOM_MODE                "Custom"
#define SETUP_INF_VENDOR                     "Vendor"
#define SETUP_INF_REINSTALL                  "Reinstall"
/*setup.inf general section keywords */
#define SETUP_INF_DIRECTORYDIALOG            "DirectoryDialog"
#define SETUP_INF_MCCDIALOG                  "MccDialog"
#define SETUP_INF_SUMMARYDIALOG              "SummaryDialog"
#define SETUP_INF_FINSIHDIALOG               "FinsihDialog"
#define SETUP_INF_INSTALLCACHEDIALOG         "InstallCacheDialog"
#define SETUP_INF_LICENSEDIALOG              "LicenseDialog"
#define SETUP_INF_WELCOMEDIALOG              "WelcomeDialog"
#define SETUP_INF_PLUGINTITLE                "PluginTitle"

#define SETUP_INF_PROGRAMFOLDER              "ProgramFolder"
#define SETUP_INF_ALL_MODES                  "ALLMODES"
#define SETUP_INF_INSTALL_DIR                "DefaultInstallDirectory"
#define SETUP_INF_REPLACEALLFILES            "ReplaceAllFiles"
#define SETUP_INF_REQUIREDOMAIN              "RequireDomain"
#define SETUP_INF_SKIPPRODUCTPAGE            "SkipProductPage"
#define SETUP_INF_SECURITYCHECK              "SecurityCheck"

/* Unix only */
#define SETUP_INF_INSTALLWHICH               "InstallWhich"
#define SETUP_INF_INSTALLSERVERS             "1"
#define SETUP_INF_INSTALLCLIENTS             "2"
#define SETUP_INF_FINISHMESSAGE              "FinishMessage"
#define SETUP_INF_BRAND                      "brand"
#define SETUP_INF_CONSOLE                    "console"

#define SETUP_DEFAULTVENDOR                  "Fedora Project"
#define SETUP_DEFAULTVERSION                 "1.0"
#define SETUP_DEFAULTBRAND                   "Fedora"
#define SETUP_DEFAULTNAME                    "Fedora Server Products"
#define SETUP_DEFAULTPROGRAMFOLDER           "Fedora Server Products"
#define SETUP_DEFAULTCONSOLE                 "Fedora Management Console"
#define SETUP_DEFAULTPLUGINTITLE             "components" 
#define SETUP_DEFAULTINSTALLDIR              "/opt/fedora/servers"
#define SETUP_DEFAULTDOWNLOADER              "Fedora Server Products Network Setup"
#define SETUP_DEFAULTDOWNLOADERVERSION       "7.1"

/* Setup global inf keywords */
#define SETUP_GLOBAL_INF_SVR_ROOT            "ServerRoot"
#define SETUP_GLOBAL_INF_LDAP_USED           "LDAPEnabled"
#define SETUP_GLOBAL_INF_LDAP_HOST           "LDAPHost"
#define SETUP_GLOBAL_INF_LDAP_PORT           "LDAPPort"
#define SETUP_GLOBAL_INF_LDAP_SUFFIX         "BaseSuffix"
#define SETUP_GLOBAL_INF_LDAP_USER           "ConfigDirectoryAdminID"
#define SETUP_GLOBAL_INF_LDAP_PASSWD         "ConfigDirectoryAdminPwd"
#define SETUP_GLOBAL_INF_ADMIN_URL           "AdminURL"
#define SETUP_GLOBAL_INF_COMPONENTS          "Components"
#define SETUP_GLOBAL_INF_FULLMACHNAME        "FullMachineName"
#define SETUP_GLOBAL_INF_LDAP_URL            "ConfigDirectoryLdapURL"
#define SETUP_GLOBAL_INF_UG_LDAP_URL         "UserDirectoryLdapURL"
#define SETUP_GLOBAL_INF_UG_ADMINID          "UserDirectoryAdminID"
#define SETUP_GLOBAL_INF_UG_ADMINPWD         "UserDirectoryAdminPwd"
#define SETUP_GLOBAL_INF_ADMIN_DOMAIN        "AdminDomain"
#define SETUP_GLOBAL_INF_SUITESPOT_USERID    "SuiteSpotUserID"
#define SETUP_GLOBAL_INF_SUITESPOT_USERGROUP "SuitespotGroup"
#define SETUP_GLOBAL_INF_LDAP_INSTALLDN      GLOBAL_INF_ADMIN_DOMAIN    /*legacy */
/* Setup messages */
#define SETUP_MSG_SUCCESS                    0x00      /* 0 */
#define SETUP_ERROR_SUCCESS                  0x00      /* 0 */
#define SETUP_SUCCESS                        0x00      /* 0 */
#define SETUP_MSG_REBOOT_REQUIRED            0x01      /* 1 */
#define SETUP_MSG_OPERATIOSETUP_ERROR        0x02      /* 2 */
#define SETUP_MSG_REINSTALL_FILES_ONLY       0x03      /* 3 */
/* Setup Error messages */
enum
{
  SETUP_ERROR=0x64,                             /* 100  */
  SETUP_ERROR_BAD_CACHE,                        /* 101  */
  SETUP_ERROR_BAD_INSTALL_PRIVILEGE,            /* 102  */
  SETUP_ERROR_SEARCH_ISIE_ROOT,                 /* 103  */
  SETUP_ERROR_CREATE_ISIE_ROOT,                 /* 104  */
  SETUP_ERROR_SEARCH_ISIE_HOST,                 /* 105  */
  SETUP_ERROR_CREATE_ISIE_HOST,                 /* 106  */
  SETUP_ERROR_UPDATE_ISIE_HOST_ACI,             /* 107  */
  SETUP_ERROR_SEARCH_ISIE_ADM_GROUP,            /* 108  */
  SETUP_ERROR_CREATE_ISIE_ADM_GROUP,            /* 109  */
  SETUP_ERROR_SEARCH_ISIE_APP,                  /* 110  */
  SETUP_ERROR_CREATE_ISIE_APP,                  /* 111  */
  SETUP_ERROR_CREATE_ISIE_ENTRY,                /* 112  */
  SETUP_ERROR_SEARCH_SIE_ROOT,                  /* 113  */
  SETUP_ERROR_CREATE_SIE_ROOT,                  /* 114  */
  SETUP_ERROR_SEARCH_SIE_GROUP,                 /* 115  */
  SETUP_ERROR_CREATE_SIE_GROUP,                 /* 116  */
  SETUP_ERROR_UPDATE_SIE_GROUP,                 /* 117  */
  SETUP_ERROR_CREATE_SIE_ENTRY,                 /* 118  */
  SETUP_ERROR_CREATE_SIE_CONFIG,                /* 119  */
  SETUP_ERROR_CREATE_SIE_TASK,                  /* 120  */
  SETUP_ERROR_INVALID_SIE_HANDLE,               /* 121  */
  SETUP_ERROR_LDAP_INIT,                        /* 122  */
  SETUP_ERROR_LDAP_BIND,                        /* 123  */
  SETUP_ERROR_LDAP_GET_DN,                      /* 124  */
  SETUP_ERROR_CREATE_SIE_GLOBAL,                /* 125  */
  SETUP_ERROR_READING_INF,                      /* 126  */
  SETUP_ERROR_UTF8_CONVERSION,                  /* 127  */
  SETUP_ERROR_MISSING_INSTALL_FILE,             /* 128  */
  SETUP_ERROR_LOADDING_PLUGINDLL,               /* 129  */
  SETUP_ERROR_INSTALLING_FILES,                 /* 130  */
  SETUP_ERROR_USER_CANCEL,                      /* 131  */
  SETUP_ERROR_INVALID_COMMAND_LINE              /* 132  */
};/**/
#define SETUP_ERROR_MAX                      0x2710  /* 10000 */
#endif /* __SETUPGLOBALS_H */


