/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** Contact:	Phong Vo (pvo@netscape.com)
**
** $Id: prodinfo.h,v 1.1.1.1 2005/07/29 22:16:29 foxworth Exp $
**
** Name: prodinfo.h
**
** $Log: prodinfo.h,v $
** Revision 1.1.1.1  2005/07/29 22:16:29  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:27  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:13  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.2  2001/11/06 01:51:31  ahakim
** XXX AOL/NSCP Copyright XXX
**
** Revision 1.1.1.1  1999/03/04 23:28:29  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.7  1998/06/05 03:38:05  pvo
** 118281 - additional code for UTF8 conversion
**
** Revision 1.1.2.6  1998/02/26 01:39:35  gbeasley
** more sie integration
**
** Revision 1.1.2.5  1997/11/26 04:42:50  pvo
** Fixed up for cross-platform compilation.
**
** Revision 1.1.2.4  1997/11/26 02:04:32  pvo
** Fixed bugs and add capability to read NVPair from a file.
**
** Revision 1.1.2.3  1997/11/07 22:43:25  pvo
** New code to support uninstallation.
**
** Revision 1.1.2.2  1997/10/06 23:50:44  pvo
** Cleaned up for NT compilation.
**
** Revision 1.1.2.1  1997/10/06 20:40:32  pvo
** Copied here from libadminutil to eliminate too much baggage due
** to dependencies.
**
**
** Revision 1.1.2.1  1997/09/11 17:58:36  pvo
** Added prodinfo.h
**
** Revision 1.2.4.1.34.1  1997/09/04 22:02:28  pvo
** Incorporate changes from 3.5.
**
** Revision 1.2.4.1.38.1  1997/07/10 02:09:46  pvo
** Added additional ADM_piTree functions.
**
** Revision 1.2.4.1  1997/03/06 22:48:12  drk
** Cleaned up to compile using C++ on HPUX.
**
** Revision 1.2  1996/10/11 04:59:29  flc
** Added list access function.
**
** Revision 1.1  1996/10/03 04:54:23  flc
** Initial checkin
**
***********************************************************************/

#ifndef prodinfo_h
#define prodinfo_h

/***********************************************************************
** Includes
***********************************************************************/

typedef struct {
  char		*var;	/* name of array */
  int		numEls;	/* how many names in array */
  char		**els;	/* names in array */
} ADM_piList;

typedef struct ADM_piSect ADM_piSect;

/*
 * Note on the format:
 *   0 : Use <sp>, = or : as delimiter (default)
 *   1 : Use = as delimiter
 *   2 : Use : as delimiter
 */
typedef struct ADM_piTree {
  int           format;
  char		*when;
  int		numVars;	/* numVars is the number of vars and vals */
  char		**vars;
  char		**vals;
  int		numBools;	/* how many boolean variables? */
  char		**boolVars;
  char		*boolVals;
  int		numLists;
  ADM_piList	*lists;
  int           numSects;
  ADM_piSect    *sects;
} ADM_piTree;

struct ADM_piSect {
  char          *name;   /* name of section */
  ADM_piTree    *sect;  /* the section */
};

/***********************************************************************
** GLOBAL FUNCTIONS:
** DESCRIPTION:
** These functions allow one to use data from files which have name value
** pairs, boolean variables, and list variables.  See
** http://poseidon/~flc/prodinfo.html for a description of the file format.
**
** EXAMPLE:
** @+@ Something that really gives the idea of how to use this set of functions
**
** RESTRICTIONS:
**
**
** RETURN CODE TABLE:
** -1:	list or variable not found
**  0:  boolean variable false or list entry not found
**  1:  boolean variable true or list entry found
***********************************************************************/

ADM_piTree *
ADM_pi_init( const char *file, int format );

ADM_piTree *
ADM_pi_init( const char *file);

void
ADM_pi_done( ADM_piTree * );

char *
ADM_pi_value( const ADM_piTree *, const char *var );

ADM_piTree *
ADM_pi_section ( const ADM_piTree *, const char *name);

ADM_piTree *
ADM_pi_addSection (ADM_piTree *, const char *name);

ADM_piTree *
ADM_pi_nsection ( const ADM_piTree *, int i);

char *
ADM_pi_sectName ( const ADM_piTree *, int i);

void
ADM_pi_rename (ADM_piTree *, const char *, const char *);

char *
ADM_pi_when( const ADM_piTree * );

int
ADM_pi_canDo( const ADM_piTree *, const char *, const char * );

int
ADM_pi_list( const ADM_piTree *, const char *name, char ***list );

void
ADM_pi_2List( ADM_piTree *, const char *name);

void
ADM_pi_setValue (ADM_piTree *tree, const char *name, const char *value);

void
ADM_pi_delValue (ADM_piTree *tree, const char *name);

void
ADM_pi_setBoolean (ADM_piTree *tree, const char *capability, const char *value);

void
ADM_pi_setCapability (ADM_piTree *tree, const char *capability, const char *instance);

void
ADM_pi_delCapability (ADM_piTree *tree, const char *capability, const char *instance);

void
ADM_pi_delList (ADM_piTree *tree, const char *name);

void
ADM_pi_delSection (ADM_piTree *, const char *name);

void
ADM_pi_setFormat (ADM_piTree *dest, int which);

void
ADM_pi_copy (ADM_piTree *dest, const ADM_piTree *src);

int
ADM_pi_write (const ADM_piTree *tree, const char *fileName);

void
ADM_pi_timeStamp(ADM_piTree *tree, const char *timestamp);

void
ADM_pi_convertEncoding(ADM_piTree *tree, const char *name, int toUTF8);

#endif /* prodinfo_h */
