/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
#include "global.h"
#include "setupdefs.h"
#include "ldapu.h"
#include "setupapi.h"
#include "setupinst.h"
#include "utf8.h"
#include "code.h"

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#if defined(AIX)
  #include <strings.h>
#endif
#if defined(SOLARIS)
  #include <sys/systeminfo.h>
#endif
#if defined(OSF1) 
  extern int statvfs(const char *path, void *);
#endif
#if defined(AIX) || defined(OSF1) || defined(HPUX) || defined(SOLARIS)
  extern int getdomainname(char *Name, int Namelen);
#endif
};

#ifdef XP_WIN32
  #include <windows.h>
  #include <regstr.h>
  #include <direct.h>
  #include <io.h>    /* For _findfirst */
#else

extern "C" {
  #include <sys/types.h>
  #include <sys/statvfs.h>
  #include <sys/socket.h>         /* socket, bind */
  #include <netinet/in.h>         /* htonl */
  #include <netdb.h>              /* gethostbyname */
  #include <arpa/inet.h>          /* inet_ntoa */
  #include <dirent.h>
  #include <assert.h>
};
#endif


#define MINPORT       1024
#define MAXPORT       65535

int iDEFAULTLOGFILESET=0;
char *szLOGFILE;

InstallLog *_setupLog = NULL;

#ifdef XP_UNIX
  #define TRUE 1
  #define FALSE 0
  #define MAX_PATH 260
// const char *PATH_RESCONF = "/etc/resolv.conf";
#endif

extern "C"
{


#ifdef XP_WIN32
static int setupDeleteRecursivelyFoundFile ( char * fullFileName, const char * path, const char * fileName );
int __cdecl setupRemoveRelative ( char * path );



int
setupRemoveRelative ( char * path )
{
  char * src;
  char * dst;

  src = path;
  dst = path;
  while (*src)
  {
    if (*src == '.')
    {
      if (*(src+1) == '.')
      {
        /* strip off the "../" */
        src += 2;

        /* back off least significant directory */
        dst--;
        if (( *dst == '\\' ) || ( *dst == '/' ))
          *dst--;
        while (dst > path)
        {
          if (( *dst == '\\' ) || ( *dst == '/' ))
            break;
          dst--;
        }
      }
      else
      {
        // remove single "."
      }

    }
    else
      *dst++ = *src++;
  }
  *dst = '\0';

  return TRUE;
}

/* if the path specified is a file name, the file is deleted
 * If the path specifies a directory, the directory is deleted
 */
NSAPI_EXTERN int
setupDeleteRecursively (const char * path )
{
  int result;
  unsigned short fileStatus;
  struct _stat buf;
  struct _finddata_t fileFound;
  long hFile;
  int retStatus = TRUE;
  char fullFileName[_MAX_PATH];
  int error;

  /* Check if statistics are valid: */
  result = _stat( path, &buf );
  if (result != 0)
    return TRUE;          // file or directory does not exist

  fileStatus = buf.st_mode & _S_IFMT;

  /* check if regular file */
  if (fileStatus & _S_IFREG)
  {
    if (remove ( path ) == -1)
    {
      error = errno;
      switch (error)
      {
      case ENOENT:
        break;

      case EACCES:
        break;

      default:
        break;
      }

      return FALSE;
    }
    return TRUE;
  }
  if ((fileStatus & _S_IFDIR) == 0)
    return FALSE;


  /* path contains a directory, delete all files recursively */
  /* Find first .c file in current directory */
  snprintf(fullFileName, sizeof(fullFileName), "%s\\*.*", path);
  if (!memchr(fulLFileName, 0, sizeof(fulLFileName))) return FALSE; /*overflow*/

  if ((hFile = _findfirst( fullFileName, &fileFound )) != -1L)
  { /* directory contain files? */
    if (!setupDeleteRecursivelyFoundFile ( fullFileName, path, fileFound.name ))
      retStatus = FALSE;

    /* Find the rest of the .c files */
    while (_findnext( hFile, &fileFound ) == 0)
    {
      if (!setupDeleteRecursivelyFoundFile ( fullFileName, path, fileFound.name ))
        retStatus = FALSE;
    }
    _findclose( hFile );
  }

  /* remove the directory, now that it is empty */
  if (_rmdir( path ) == -1)
    retStatus = FALSE;
  return retStatus;
}



static int
setupDeleteRecursivelyFoundFile ( char * fullFileName, const char * path, const char * fileName )
{
  if (strcmp ( fileName, "." ) == 0)
    return TRUE;

  if (strcmp ( fileName, ".." ) == 0)
    return TRUE;

  snprintf(fullFileName, sizeof(fullFileName), "%s\\%s", path, fileName);
  if (!memchr(fulLFileName, 0, sizeof(fulLFileName))) return FALSE; /*overflow*/

  return setupDeleteRecursively ( fullFileName );
}
#endif

/* GetNextFileInDirectory - gets next file in the directory
 * Set hFile to zero, when you call it. The routine returns the
 * next value for hFile. When the routine returns NULL, there is
 * no more files
 *
 */
NSAPI_EXTERN int
setupGetNextFileInDirectory (long hFile, const char * path, char * lpFileName, int lpFileNameLen )
{
#ifdef XP_UNIX
  static DIR *dirp = NULL;
  struct dirent *dp = NULL;
  NSString filename;

  if (hFile == 0)
  {
    if (dirp)
    {
      closedir(dirp);
    }

    dirp = opendir(path);
  }

  if (dirp == NULL)
  {
    return 0;
  }

  while ((dp = readdir(dirp)) != NULL)
  {
    if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, ".."))
    {
      snprintf(lpFileName, lpFileNameLen, "%s", dp->d_name);
      if (!memchr(lpFileName, 0, lpFileNameLen)) return 0; /*overflow*/
      return 1;
    }
  }

  closedir(dirp);
  dirp = NULL;

  return 0;
#else
  int result;
  unsigned short fileStatus;
  struct _stat buf;
  struct _finddata_t fileFound;
  char fullFileName[_MAX_PATH];
  static long currentFile = 0;

  if (hFile == 0)
  {
    if (currentFile)
    {
      _findclose(currentFile);
      currentFile = 0;
    }

    /* Check if statistics are valid: */
    result = _stat( path, &buf );
    if (result != 0)
      return 0;         // file or directory does not exist

    fileStatus = buf.st_mode & _S_IFMT;
    if ((fileStatus & _S_IFDIR) == 0)
      return 0;


    /* path contains a directory, delete all files recursively */
    /* Find first .c file in current directory */
    snprintf(fullFileName, sizeof(fullFileName), "%s\\*.*", path);
    if (!memchr(fullFileName, 0, sizeof(fullFileName))) return 0; /*overflow*/

    if ((hFile = _findfirst( fullFileName, &fileFound )) == -1L)
      return 0;

    if (( strcmp ( fileFound.name , "." ) != 0)
        && ( strcmp ( fileFound.name , ".." ) != 0))
    {
      snprintf(lpFileName, lpFileNameLen, "%s", fileFound.name );
      if (!memchr(lpFileName, 0, lpFileNameLen)) return 0; /*overflow*/

      currentFile = hFile;
      return hFile;
    }
  }

  /* Find the rest of the .c files */
  while (_findnext( hFile, &fileFound ) == 0)
  {
    if (( strcmp ( fileFound.name , "." ) != 0)
        && ( strcmp ( fileFound.name , ".." ) != 0))
    {
      snprintf(lpFileName, lpFileNameLen, "%s", fileFound.name );
      if (!memchr(lpFileName, 0, lpFileNameLen)) return 0; /*overflow*/

      currentFile = hFile;
      return hFile;
    }
  }

  _findclose( hFile );
  currentFile = 0;

  return 0;
#endif
}


/*---------------------------------------------------------------------------*\
 *
 * Function:  setupGetNextSubDirectory
 *
 *  Purpose:  Gets next sub directory in the path
 *
 *    Input:
 *          hFile: set to zero first time called; use return value for subsequent calls
 *          path: directory containing sub directories
 *          lpSubDirectoryName: buffer to store sub directory name
 *          lpSubDirectoryNameLen: length of buffer lpSubDirectoryName
 *          lpSubDirectoryPrefix: chars to exactly match begining of directory name
 *
 *  Returns:
 *          hFile to be used on subsequent call (0, if no more directories)
 *
 * Comments:
\*---------------------------------------------------------------------------*/
NSAPI_EXTERN int
setupGetNextSubDirectory( long hFile, char *path,
                          char *lpSubDirectoryName, int lpSubDirectoryNameLen,
                          char *lpSubDirectoryPrefix )
{
#ifdef XP_UNIX
  static DIR *dirp = NULL;
  struct dirent *dp;
  struct stat fi;
  NSString filename;

  if (hFile == 0)
  {
    if (dirp)
    {
      closedir(dirp);
    }
    dirp = opendir(path);
  }

  if (dirp == NULL)
  {
    return 0;
  }

  while ((dp = readdir(dirp)) != NULL)
  {
    if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, ".."))
    {
      filename = path;
      filename = filename + "/";
      filename = filename + dp->d_name;
    
      if (stat(filename, &fi) == 0 && S_ISDIR(fi.st_mode))
      {
        if (lpSubDirectoryPrefix == NULL ||
            !strncmp(dp->d_name, lpSubDirectoryPrefix, strlen(lpSubDirectoryPrefix)))
        {
          snprintf(lpSubDirectoryName, lpSubDirectoryNameLen,
                             "%s", dp->d_name);
          if (!memchr(lpSubDirectoryName, 0, lpSubDirectoryNameLen)) return 0; /*overflow*/
          return 1;
        }
      }
    }
  }

  closedir(dirp);
  dirp = NULL;

  return 0;
#else
  int result;
  unsigned short fileStatus;
  struct _stat buf;
  char * subDirectoryPrefix;
  char * p;
  char fullFileName[_MAX_PATH];
  BOOL bSubDirectoryFound;

  do
  {
    hFile = setupGetNextFileInDirectory ( hFile, path, lpSubDirectoryName, lpSubDirectoryNameLen );
    if (hFile == 0)
      return 0;

    /* Check if file is a directory */
    snprintf(fullFileName, sizeof(fullFileName), "%s\\%s",
                      path, lpSubDirectoryName);
    if (!memchr(fullFileName, 0, sizeof(fullFileName))) return 0; /*overflow*/

    result = _stat( fullFileName, &buf );
    if (result == 0)
    {
      fileStatus = buf.st_mode & _S_IFMT;
      if ((fileStatus & _S_IFDIR) == _S_IFDIR)
      {

        /* check if sub directory matches prefix */
        bSubDirectoryFound = TRUE;
        if (lpSubDirectoryPrefix)
        {
          p = lpSubDirectoryName;
          subDirectoryPrefix = lpSubDirectoryPrefix;
          while (*subDirectoryPrefix)
          {
            if (*subDirectoryPrefix++ != *p++)
            {
              bSubDirectoryFound = FALSE;
              break;
            }
          }
        }
        if (bSubDirectoryFound)
          return hFile;
      }
    }
  } while (hFile);

  return 0;       // no more sub directories
#endif
}

int setupDeleteDir(const char *path)
{
#ifdef XP_WIN32
   return setupDeleteRecursively(path);
#else
   DIR *dirp;
   struct dirent *dp;
   Bool rc = True;
   struct stat fi;
   NSString filename;
   int err = 0;

   dirp = opendir(path);

   if (dirp == NULL)
   {
      err = unlink(path);
      return err;
   }

   while ((dp = readdir(dirp)) != NULL)
   {
      if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, ".."))
      {
         filename = path;
         filename = filename + "/";
         filename = filename + dp->d_name;

         if (stat(filename, &fi) == 0 && !S_ISDIR(fi.st_mode))
         {
            unlink(filename);
         }
         else
         {
            setupDeleteDir (filename);
            rmdir(filename);
         }
      }
   }

   err = rmdir(path);

   closedir(dirp);

   return err;
#endif
}


  /*********************************************************************
  **
  ** FUNCTION:    unescape_ldap_basedn
  ** DESCRIPTION:
  **   Convert an ldap url from db format to the correct format.
  **   Copied from ldaputil!
  **
  ** INPUTS:      str  - the ldap url in db format
  ** OUTPUTS:     str  - the ldap url in correct format
  ** RETURN:      none
  ** SIDE EFFECTS:
  **      the input ldap url is modified inline.
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************
  */

  void
  unescape_ldap_basedn(char *str)
  {
    if (strchr(str, '%'))
    {
      register int x = 0, y = 0;
      int l = strlen(str);
      char digit;

      while (x < l)
      {
        if ((str[x] == '%') && (x < (l - 2)))
        {
          ++x;
          digit = (str[x] >= 'A' ?
                   ((str[x] & 0xdf) - 'A') + 10 : (str[x] - '0'));
          digit *= 16;

          ++x;
          digit += (str[x] >= 'A' ?
                    ((str[x] & 0xdf) - 'A') + 10 : (str[x] - '0'));

          str[y] = digit;
        }
        else
        {
          str[y] = str[x];
        }
        x++;
        y++;
      }
      str[y] = '\0';
    }
  }



/*********************************************************************
**
** FUNCTION:    setupGetDefaultLdapInfo
** replaced: getDefaultLdapInfo
** DESCRIPTION:
**   Tries to determine an existing ldap url to be used by the
**   to be installed server. The routine looks into the current
**   server root to see whether ldap information is already stored
**   there.
**
** INPUTS:
**   sroot                      - server root
**   ldapURL                    - ldap url to be returned
**   ldapUser                   - optional bind user id to be returned
**
** OUTPUTS:     ldapURL, ldapUser, bindDN, bindPwd
** RETURN:      none
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**      Returned memory is allocated for char ** parameters
**********************************************************************
*/


NSAPI_EXTERN void
setupGetDefaultLdapInfo(const char *sroot, char ** ldapURL, char ** ldapUser,
                      char **adminDomain)
{
  char tstr[BIG_BUF];
  snprintf(tstr, sizeof(tstr), "%s/%s", sroot, SETUP_LDAP_CONF);
  if (!memchr(tstr, 0, sizeof(tstr))) return; /*overflow*/
  if (ldapURL)
  {
    *ldapURL = setupGetFileString("ConfigURL", " ", tstr);
    if (*ldapURL == NULL)
    {
      char dbswith[BIG_BUF];
      snprintf(dbswith, sizeof(dbswith), "%s/%s", sroot, SETUP_DEFAULT_LDAPSWITCH);
      if (!memchr(dbswith, 0, sizeof(dbswith))) return; /*overflow*/
      *ldapURL = setupGetFileString("directory default", " ", dbswith);
    }

    if (*ldapURL && ldap_is_ldap_url(*ldapURL))
    {
      unescape_ldap_basedn(*ldapURL);
    }
  }

  if (ldapUser)
  {
    *ldapUser = setupGetFileString("admnm", " ", tstr);  
  }

  if (adminDomain)
  {
    *adminDomain = setupGetFileString(DS_ADMIN_DOMAIN, ":", tstr);
    if (*adminDomain == NULL)
    {
      char *s = NULL;
      snprintf(tstr, sizeof(tstr), "%s/%s", sroot, SETUP_DS_CONF);
      if (!memchr(tstr, 0, sizeof(tstr))) return; /*overflow*/
      s = setupGetFileString(DS_ADMIN_DOMAIN, ":", tstr);
      if (s)
      {
        *adminDomain = setupStripAdminDomainDN(s);
        setupFree(s);
      }
    }
  }

  return;
}

/*********************************************************************
**
** FUNCTION:    setupSetDefaultLdapInfo
** replaced: getDefaultLdapInfo
** DESCRIPTION:
**   Set Config Ldap information to ldap.conf. Caller needs to fill the  
**   value to each argument.  NULL means not to change the existing value.
**
** INPUTS:
**   sroot                      - server root
**   ldapURL                    - config LDAP URL
**   ldapUser                   - admin user to access ldapURL
**   adminDoamin                - local admin domain
**
** OUTPUTS:     none
** RETURN:      none
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**      Returned memory is allocated for char ** parameters
**********************************************************************
*/


NSAPI_EXTERN void
setupSetDefaultLdapInfo(const char *sroot, char * ldapURL, char * ldapUser,
                      char *adminDomain)
{
  char tstr[BIG_BUF];
  char *oldLines[2];
  char *newLines[2];
  char buffer[100];

  if (ldapURL)
  {

    snprintf(tstr, sizeof(tstr), "%s/%s", sroot, SETUP_DEFAULT_LDAPSWITCH);
    if (!memchr(tstr, 0, sizeof(tstr))) return; /*overflow*/

    oldLines[1] = NULL;
    newLines[1] = NULL;
    snprintf(buffer, sizeof(buffer), "directory default %s", ldapURL);
    if (!memchr(buffer, 0, sizeof(buffer))) return; /*overflow*/

    oldLines[0] = (char *)"directory default ";
    newLines[0] = buffer; 
    setupFileUpdateLines(oldLines, newLines, tstr);
  }

  if (ldapUser)
  {       
    snprintf(tstr, sizeof(tstr), "%s/%s", sroot, SETUP_LDAP_CONF);
    if (!memchr(tstr, 0, sizeof(tstr))) return; /*overflow*/

    oldLines[1] = NULL;
    newLines[1] = NULL;
    snprintf(buffer, sizeof(buffer), "admnm  %s", ldapUser);
    if (!memchr(buffer, 0, sizeof(buffer))) return; /*overflow*/

    oldLines[0] = (char *)"admnm";
    newLines[0] = buffer;                     
    setupFileUpdateLines(oldLines, newLines, tstr);
  }

  if (adminDomain)
  {
    snprintf(tstr, sizeof(tstr), "%s/%s", sroot, SETUP_DS_CONF);
    if (!memchr(tstr, 0, sizeof(tstr))) return; /*overflow*/

    oldLines[1] = NULL;
    newLines[1] = NULL;
    snprintf(buffer, sizeof(buffer), "%s:%s", DS_ADMIN_DOMAIN, adminDomain);
    if (!memchr(buffer, 0, sizeof(buffer))) return; /*overflow*/

    oldLines[0] = (char *) DS_ADMIN_DOMAIN;
    newLines[0] = buffer;                     
    setupFileUpdateLines(oldLines, newLines, tstr);
  }

  return;
}




  /*********************************************************************
  **
  ** FUNCTION:    setupFormConfigLdapURL
  ** Replaced: formConfigLdapURL
  ** DESCRIPTION:
  **   Take LdapURL and change it Configuration Ldap URL
  **   Configuration Ldap URL is hardcoded to use DEFAULT_ROOT_DN suffix.
  **
  ** INPUTS:
  **   ldapURL                    - ldap url
  **
  ** RETURN:
  **   On success, it returns config ldap URL
  **   NULL:  if ldapURL doesn't contain "//"
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **      Need to free the return memory
  **********************************************************************
  */

  NSAPI_EXTERN char *
  setupFormConfigLdapURL(const char *ldapURL)
  {
    char *configURL = NULL;
    char *uri, *p;
    int configUrlFullLen;
    int configUrlLen;

    if (ldapURL == NULL)
      return NULL;

    configUrlFullLen = strlen(ldapURL) + strlen(DEFAULT_ROOT_DN) + 2;
    configURL = (char *) malloc(configUrlFullLen);
    if (NULL == configURL)
        return NULL;
    configURL[0] = '\0';

    p = strchr((char *)ldapURL, '/');

    if ((p != NULL) && (*(p+1) == '/'))
    {
      /*
       *  get ldap://host:port portion of ldapURL
       */
      if ((uri = strchr(p + 2, '/')) != NULL)
      {
        configUrlLen = uri - ldapURL;
        if (configUrlLen <= 0 ||
            configUrlLen > configUrlFullLen) { /* never happens */
            free(configURL);
            return NULL;
        }
        strncpy(configURL, ldapURL, configUrlLen);
        configURL[configUrlLen] = '\0';
      }
      else
      {
        strncpy(configURL, ldapURL, configUrlFullLen);
      }

      /*
       * Append DEFAULT_ROOT_DN to hostPortURL
       */
      if (*configURL != '\0')
      {
        char *configp;
        configUrlLen = strlen(configURL);
        configp = configURL + configUrlLen;
        snprintf(configp, configUrlFullLen - configUrlLen,
                       "/%s", DEFAULT_ROOT_DN);
        if (!memchr(configp, 0, configUrlFullLen - configUrlLen)) {
            free(configURL);
            return NULL;
        }
        return configURL;
      }
    }

    return configURL;
  }


  /*********************************************************************
  **
  ** FUNCTION:    setupStripConfigLdapURL
  ** Replaces:    stripConfigLdapURL
  ** DESCRIPTION:
  **   Take Configuration Ldap URL and remove DEFAULT_ROOT_DN suffix.
  **
  ** INPUTS:
  **   configURL       - Configuration Ldap URL or ldap url
  **
  ** RETURN:
  **   On success, it returns ldapURL ldap://host:port/
  **   NULL:  if unable to form ldapURL
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **      Need to free the return memory
  **********************************************************************
  */

  NSAPI_EXTERN char *
  setupStripConfigLdapURL(const char *configURL)
  {
    char *ldapURL = NULL;
    char *uri, *p;
    int len;
    int ldapURLLen;

    if (configURL == NULL)
      return NULL;

    ldapURLLen = strlen(configURL)+2;
    ldapURL = (char *) malloc(ldapURLLen);
    if (NULL == ldapURL)
        return NULL;
    ldapURL[0] = '\0';

    p = strchr((char *)configURL, '/');

    if ((p != NULL) && (*(p+1) == '/'))
    {
      /*
       *  get ldap://host:port/ portion of configldapURL
       */
      if ((uri = strchr(p + 2, '/')) != NULL)
      {
        uri++;
        len = uri - configURL;
        if (len < 0) { /* never happens */
          free(ldapURL);
          return NULL;
        }
        strncpy(ldapURL, configURL, len);
        ldapURL[len] = '\0';
      }
      else
      {
        strncpy(ldapURL, configURL, ldapURLLen - 2);
        strncat(ldapURL, "/", 1);
      }
    }

    return ldapURL;
  }




  /*********************************************************************
  **
  ** FUNCTION:    setupIsValidLdapServer
  ** Replaces     IsValidLdapServer
  ** DESCRIPTION:
  **   validates ldap server
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **      Need to free the return memory
  **********************************************************************/


  NSAPI_EXTERN int
  setupIsValidLdapServer(const char* pszHost, int nPort, const char* pszSuffix,
                       const char* pszUserDN, const char* pszPassword)
  {
    int  bResult = FALSE;
    int   nRetVal;
    LDAP* ld;
    LDAPMessage* msg;
    struct timeval tv = { 10, 0};

    /*
    ** Attempt to connect and bind to the directory.  If succesful, search
    ** from the given base for all object classes, with a scope of base.
    ** This should tell us if the base suffix provided is valid or not.
    */

    if (sizeof(pszHost) == 0)
    {
      return bResult;
    }

    ld = ldap_init(pszHost, nPort);

    int nVer = LDAP_VERSION3;
    // ldap_set_option (ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF);

    // set protocol version to ldapv3.  Required for smart referral provided by Hin 
    ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &nVer); 

    if (ld != NULL && ldap_simple_bind_s(ld, pszUserDN, pszPassword) == LDAP_SUCCESS)
    {
      nRetVal = ldap_search_st(ld, pszSuffix, LDAP_SCOPE_BASE,
                               OBJECT_CLASS_FILTER, NULL, FALSE, &tv, &msg);


      //if you are doing a simple bind then you may not have access so
      //assume that since LDAP_SUCCES then the suffix is there.
      //if (nRetVal == LDAP_SUCCESS && ldap_count_entries(ld, msg) > 0)
      if (nRetVal == LDAP_SUCCESS)
      {
        bResult = TRUE;
      }

      ldap_msgfree(msg);
      ldap_unbind(ld);
    }

    return bResult;
  }

  /*********************************************************************
  **
  ** FUNCTION:    setupIsValidLdapUser
  ** Replaced:    IsValidLdapUser
  ** DESCRIPTION:
  **   validates ldap user
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      if bReturnDN set to true modifies psaUser to full DN
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **      Need to free the return memory
  **********************************************************************/


  NSAPI_EXTERN int
  setupIsValidLdapUser(const char* pszHost, int nPort, const char* pszSuffix,
                     char** pszUser, const char* pszPassword, int bReturnDN)
  {
    int  bResult = FALSE;
    LdapErrorCode err;
    Ldap *ldap;
    const char* pszDN;
    char ldapURL[MAX_PATH];

    if (sizeof(pszHost) == 0)
    {
      return bResult;
    }

    /*
    ** Attempt to connect and bind to the directory as the specified user.  This
    ** will at least tell us that the user exists.  What it won't tell us, is
    ** whether or not the user has the correct access privileges to the directory.
    */
    snprintf(ldapURL, sizeof(ldapURL), "ldap://%s:%d/%s", pszHost, nPort, pszSuffix);
    if (!memchr(ldapURL, 0, sizeof(ldapURL))) return bResult; /* overflow */

    err = createLdap(&ldap, ldapURL, *pszUser, pszPassword, NULL, NULL);

    if (err == OKAY)
    {
      if (bReturnDN)
      {
        int pszUserLen;
        pszDN = userDN(ldap);
        setupFree(*(pszUser));
        pszUserLen = strlen(pszDN) + 2;
        *(pszUser)= (char *) malloc(pszUserLen);
        if (NULL == *pszUser)
            return bResult;
        strncpy(*(pszUser), pszDN, pszUserLen);
      }
      destroyLdap(ldap);
      bResult=TRUE;
    }
    return bResult;
  }

  /*********************************************************************
  **
  ** FUNCTION:    setupParseLdapUrl
  ** Replaced:    ParseLdapUrl
  ** DESCRIPTION:
  **   parse ldap url
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **      Need to free the return memory
  **********************************************************************/

  NSAPI_EXTERN int
  setupParseLdapUrl(char * szldapURL, char ** szldapDomain, unsigned int *uldapPort, char ** szldapDN)
  {
    LDAPURLDesc *ludpp = NULL;
    char *ldapURL = NULL;
    char *uri, *p;
    int ldapURLLen;

    if (!szldapURL)
    {
      return FALSE;
    }
    ldapURLLen = strlen(szldapURL) + 2;
    ldapURL = (char *) malloc(ldapURLLen);
    if (NULL == ldapURL)
           return FALSE;
    p = strchr(szldapURL, '/');
    if ((p != NULL) && (*(p+1) == '/'))
    {
      if ((uri = strchr(p + 2, '/')) != NULL)
      {
        strncpy(ldapURL, szldapURL, ldapURLLen);
      }
      else
      {
        strncpy(ldapURL, szldapURL, ldapURLLen - 1);
        strncat(ldapURL, "/", 1);
      }
    }


    *szldapDomain = *szldapDN = NULL; /* possible memory leak */
    *uldapPort = 0;

    if (!(ldapURL) || !ldap_is_ldap_url(ldapURL))
    {
      return FALSE;
    }

    if (ldap_url_parse(ldapURL, &ludpp) == 0)
    {
      int len = strlen(ludpp->lud_host) + 2;
      *(szldapDomain) = (char *) malloc(len);
      if (NULL == *(szldapDomain))
           return FALSE;
      strncpy(*(szldapDomain), ludpp->lud_host, len);
      if (ludpp->lud_dn)
      {
        int len = strlen(ludpp->lud_dn) + 2 + 2;
        *(szldapDN) = (char *) malloc(len);
        if (NULL == *(szldapDN))
             return FALSE;
        strncpy(*(szldapDN), ludpp->lud_dn, len);
      }
      *uldapPort = ludpp->lud_port;
    }
    else return FALSE;

    return TRUE;
  }

  /*********************************************************************
  **
  ** FUNCTION:    setupFree
  ** Replaced:    setupetupFree
  ** DESCRIPTION:
  **   free allocated memory that was allocated by libaryies functions
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/



  NSAPI_EXTERN void
  setupFree(char *src)
  {
    if (src == NULL) return;

#ifdef XP_WIN32
    free(src);
#else
    free(src);
#endif
    return;
  }

  NSAPI_EXTERN void *
  setupMalloc(unsigned int size)
  {
     return malloc(size);
  }
  NSAPI_EXTERN char *
  setupStrdup(char *src)
  {
    if (!(src))
    {
       return NULL;
    }
    return strdup(src);
  }

  /*********************************************************************
  **
  ** FUNCTION:    setupInsertPluginLdifEntries
  ** Replaced:    InsertPluginLdifEntries
  ** DESCRIPTION:
  **   insert ldif into subTree
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/


  NSAPI_EXTERN int
  setupInsertPluginLdifEntries(const char *ldapURL, const char *subTreeDN,
                             const char *ldapUser, const char *ldapPwd, const char *ldifFile)
  {
    Ldap* ldap;
    LdapErrorCode err;

    err = createLdap(&ldap, ldapURL, ldapUser, ldapPwd, NULL, NULL);
    if (err != OKAY)
      return err;

    err = insertLdifEntries (ldap, subTreeDN, ldifFile, NULL);

    destroyLdap (ldap);

    return err;
  }

/*********************************************************************
**
** FUNCTION:    setupInsertPluginSchemaEntries
** Replaced:    InsertPluginLdifEntries
** DESCRIPTION:
**   insert schema into cn=schema
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/


NSAPI_EXTERN int
setupInsertPluginSchemaEntries(const char *ldapURL, const char *ldapUser, const char *ldapPwd, const char *schemaFile)
{
  Ldap* ldap;
  LdapErrorCode err;

  err = createLdap(&ldap, ldapURL, ldapUser, ldapPwd, NULL, NULL);
  if (err != OKAY)
    return err;

  err = insertSchemaEntries (ldap, schemaFile);

  destroyLdap (ldap);

  return err;
}



  /*********************************************************************
  **
  ** FUNCTION:    setupFormAdminDomainDN
  ** Replaced:    FormAdminDomainDN
  ** DESCRIPTION:
  **   forms the full Admin Domain DN
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/

  NSAPI_EXTERN char *
  setupFormAdminDomainDN(const char *domain)
  {
    char *domainDN;
    int domainDNLen;

    if (domain == NULL)
      return NULL;

    domainDNLen = strlen(domain) + strlen(DEFAULT_ROOT_DN) + 10;
    domainDN = (char *) malloc (domainDNLen);
    if (NULL == domainDN)
        return NULL;
    if (strpbrk(domain, "=,") != NULL)
    {
      strncpy(domainDN, domain, domainDNLen);
    }
    else
    {
      snprintf(domainDN, domainDNLen, "ou=%s, %s", domain, DEFAULT_ROOT_DN);
      if (!memchr(domainDN, 0, domainDNLen)) return NULL; /*overflow*/
    }

    return domainDN;

  }
  /*********************************************************************
  **
  ** FUNCTION:    setupStripAdminDomainDN
  ** Replaced:    StripAdminDomainDN
  ** DESCRIPTION:
  **   strips the Admin Domain DN to just domain name
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/


  NSAPI_EXTERN char *
  setupStripAdminDomainDN(const char *domainDN)
  {
    char *domain;
    int domainLen;
    char *start, *end;

    if (domainDN == NULL)
      return NULL;

    domainLen = strlen(domainDN) + 1;
    domain = (char *) malloc (domainLen);
    if (NULL == domain)
        return NULL;
    strncpy(domain, domainDN, domainLen);

    if ((start = strpbrk((char *)domainDN, "=,")) != NULL)
    {
      start ++;
      end = strpbrk(start, "=,");
      if (end != NULL)
      {
        int delta = end - start; /* delta < domainLen */
        if (delta < 0) {
          free(domain);
          return NULL;
        }
        strncpy(domain, start, delta);
        domain[delta] = '\0';
      }
    }

    return domain;

  }

/*********************************************************************
**
** FUNCTION:    setupLog
** Replaced:    LogData
** DESCRIPTION:
**   set logFile so that LogData, can be passed in NULL, and users
**   can write to into common file log that was set by the installer
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/

NSAPI_EXTERN int setupSetLogFile(const char *pszFile)
{
   if (_setupLog == NULL)
      _setupLog = new InstallLog(pszFile);

   if (_setupLog && _setupLog->isValid())
      return True;
   else
      return False;
#if 0
  FILE *fp = NULL;

  if (pszFile == NULL)
  {
    return FALSE;
  }
  szLOGFILE = strdup(pszFile);
  fp = fopen(szLOGFILE, "a+");
  if (fp)
  {
    iDEFAULTLOGFILESET = 1;
    fclose(fp);
  }
  else return FALSE;

  return TRUE;
#endif
}


/*********************************************************************
**
** FUNCTION:    setupLog
** Replaced:    LogData
** DESCRIPTION:
**   LogData, logs data to a file, if pszFile is NULL than lod data
**   put file info into common file log that was set by the installer
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/
NSAPI_EXTERN int setupLogMessage(const char *level, const char *who, const char *msg, ...)
{
   char buf[BIG_BUF];
   va_list arg;

   if (_setupLog)
   {
      int rval;
      va_start (arg, msg);
      rval = vsnprintf(buf, sizeof(buf), msg, arg);
      va_end(arg);
      if (rval >= sizeof(buf)) return False; /* overflow */

      _setupLog->logMessage(level, who, buf);

      return True;
   }
   return False;
}


NSAPI_EXTERN int setupLog(const char *pszFile, const char *pszText)
{
   if (pszFile == NULL)
      setupLogMessage(NULL, NULL, pszText);
   else
      InstallLog::log(pszFile, pszText);

   return True;
}


  /*********************************************************************
  **
  ** FUNCTION: setupIsDirEmpty
  ** Replaced: IsDirEmpty
  **
  ** DESCRIPTION: checks to see if directory empty
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/

  NSAPI_EXTERN int setupIsDirEmpty(const char * pszPath)
  {
#ifdef XP_WIN32
    int iReturn = TRUE;
    char szFileSpec[MAX_PATH];
    WIN32_FIND_DATA fd;
    HANDLE ff;

    if (!(pszPath))
    {
      setupLog(NULL, "setupIsDirEmpty failed: passed in NULL parameter for directory Name");
      return FALSE;
    }
    snprintf(szFileSpec, sizeof(szFileSpec), "%s\\*", pszPath);
    if (!memchr(szFileSpec, 0, sizeof(szFileSpec))) return FALSE; /*overflow*/

    ff = FindFirstFile(szFileSpec, &fd);
    if (ff != INVALID_HANDLE_VALUE)
    {
      do
      {
        if (strcmp(fd.cFileName, ".") && strcmp(fd.cFileName, ".."))
        {
          iReturn = FALSE;
          break;
        }
      } while (FindNextFile(ff, &fd));
      FindClose(ff);
    }

    return (iReturn);
#else
    DIR *dirp;
    struct dirent *dp;
    Bool rc = True;

    dirp = opendir(pszPath);
    while ((dp = readdir(dirp)) != NULL)
    {
      if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, ".."))
      {
        rc = False;
        break;
      }
    }
    closedir(dirp);
    return rc;
#endif

  }

  /*********************************************************************
  **
  ** FUNCTION: setupFileExist
  ** Replaced: FileExist
  **
  ** DESCRIPTION: returns 1 if file exists.
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN int setupFileExists(const char * pszFileName)
  {
    struct stat fi;

    if ((stat (pszFileName, &fi) != -1) && ((fi.st_mode & S_IFDIR) == 0))
    {
      return True;
    }
    else 
    {
      return False;
    }
  }


  /*********************************************************************
  **
  ** FUNCTION: setupDirExists
  ** Replaced: DirExists
  **
  ** DESCRIPTION: checks to see if directory exists
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN int setupDirExists(const char * pszDirName)
  {
#ifdef XP_WIN32
    int dwAttrs;
    if (!(pszDirName))
    {
      setupLog(NULL, "DirExists failed: passed in NULL parameter for directory");
      return FALSE;
    }
    dwAttrs = GetFileAttributes(pszDirName);
    if ((dwAttrs != 0xFFFFFFFF) && (dwAttrs & FILE_ATTRIBUTE_DIRECTORY))
    {
      return TRUE;
    }
    return FALSE;
#else
    struct stat fi;

    if (stat (pszDirName, &fi) == -1 || !S_ISDIR(fi.st_mode))
    {
      return False;
    }
    else
    {
      return True;
    }
#endif
  }

  /*********************************************************************
  **
  ** FUNCTION: setupCreateDir
  ** Replaced: CreateDir
  **
  ** DESCRIPTION: creates a directory
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   NT:  TRUE or FALSE
  **   UNIX:
  **       0 : OK
  **      -1 : Can't create directory
  **      -2 : input exists and is not a directory
  **      -3 : Can't write to directory
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **
  **********************************************************************/
  NSAPI_EXTERN int setupCreateDir(const char * pszDirName, int mode)
  {
#ifdef XP_WIN32
    char *pszDelim;
    char szDirName[MAX_PATH];
    char szTmpDir[MAX_PATH] = "";
    int  cbLen;
    int  iResult = TRUE;

    if (!(pszDirName))
    {
      setupLog(NULL, "CreateDir failed: passed in NULL parameter for directory");
      return FALSE;
    }

    if (strlen(pszDirName) >= sizeof(szDirName))
    {
      setupLog(NULL, "CreateDir failed: passed in too long directory");
      return FALSE;
    }

    snprintf(szDirName, sizeof(szDirName), "%s", pszDirName);
    if (!memchr(szDirName, 0, sizeof(szDirName))) return False; /*overflow*/
    pszDelim = szDirName;
    while ((pszDelim = strchr(pszDelim, '/')) != NULL)
    {
      *pszDelim = '\\';
    }

    pszDelim = szDirName;
    while ((pszDelim = strchr(pszDelim, '\\')) != NULL)
    {
      cbLen = (pszDelim - szDirName);
      strncpy(szTmpDir, szDirName, cbLen);
      szTmpDir[cbLen] = '\0';
      CreateDirectory(szTmpDir, NULL);
      pszDelim++;
    }

    if (!CreateDirectory(szDirName, NULL))
    {
      char szLog[MAX_PATH+50];
      iResult = FALSE;
      snprintf(szLog, sizeof(szLog),
                      "setupCreateDir failed to create '%s'", pszDirName);
      if (!memchr(szLog, 0, sizeof(szLog))) return FALSE; /*overflow*/
      setupLog(NULL, szLog);
    }
    return iResult;
#else
    struct stat fi;
    char *s;
    char *t;

    s = strdup(pszDirName);
    t = s + 1;

    while (1)
    {
      t = strchr(t, '/');

      if (t)
        *t = '\0';
      if (stat(s, &fi) == -1)
      {
        if (mkdir(s, mode) == -1)
        {
          return -1;
        }
      }
      else if (!S_ISDIR(fi.st_mode))
      {
        return -2;
      }
      if (t)
        *t++ = '/';
      else
        break;
    }

    if (access(pszDirName, W_OK) == -1)
    {
      return -3;
    }
    return 0;
#endif
  }
  /*********************************************************************
  **
  ** FUNCTION: setupConvertsPathSlashes
  ** Replaced: ConvertsPathSlashes
  **
  ** DESCRIPTION: converts slashes TRUE UNIX to NT, FALSE NT to UNIX,
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN int   setupConvertPathSlashes(const char *  lpszPath, char *  lpszNewPath, int bType )
  {
    if (lpszPath == NULL)
      return FALSE;

    /* create reverse slashes and escape them */
    while (*lpszPath)
    {
      if ((*lpszPath == '\\') || (*lpszPath == '/'))
      {
        if (bType)
          *lpszNewPath = '\\';
        else
          *lpszNewPath = '/';
      }
      else
        *lpszNewPath = *lpszPath;
      lpszPath++;
      lpszNewPath++;
    }
    return TRUE;
  }


  /*********************************************************************
  **
  ** FUNCTION: setupGetHostName
  ** Replaced: GetHostName note behaviour change, setupGetHostName returns the host name
  **
  ** DESCRIPTION: returns the host name
  **
  **
  ** INPUTS: none
  **
  ** RETURN:
  **   host name or null if unable to get host name
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **  need to free memory by calling setupFree
  **********************************************************************/
  NSAPI_EXTERN char *setupGetHostName()
  {
#ifdef XP_WIN32
    char *szHostName = NULL;
    WSADATA wsaData;
    if (WSAStartup(MAKEWORD(1, 1), &wsaData) == 0)
    {
      char szHost[MAX_PATH];

      if (gethostname(szHost, sizeof(szHost)) == 0)
      {
        if (strchr(szHost, '.'))
        {
          szHostName = strdup(szHost);
        }
        else
        {
          char szRegKey[MAX_PATH];
          HKEY hKey;

          snprintf(szRegKey, sizeof(szRegKey), REGSTR_PATH_SERVICES "\\Tcpip\\Parameters");
          if (!memchr(szRegKey, 0, sizeof(szRegKey))) return NULL; /*overflow*/
          if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
          {
            unsigned char szDomain[MAX_PATH];
            DWORD dwSize = sizeof(szDomain);

            if (RegQueryValueEx(hKey, "Domain", 0, NULL, szDomain, &dwSize) == ERROR_SUCCESS)
            {
              char pszHostName[2*MAX_PATH];
              snprintf(pszHostName, sizeof(pszHostName), "%s.%s", szHost, szDomain);
              if (!memchr(pszHostName, 0, sizeof(pszHostName))) return NULL; /*overflow*/
              szHostName = strdup(pszHostName);
            }
            else
            {
              szHostName = strdup(szHost);
            }
            RegCloseKey(hKey);
          }
          else
          {
            szHostName = NULL;
          }
        }
      }
      else
      {
        szHostName = NULL;
      }
      WSACleanup();
    }
    else
    {
      szHostName = NULL;
    }

    return szHostName;
#else
    char * hn;
    char * dn = setupGetNetworkDomain();
    int hnlen;

    hnlen = sizeof(SML_BUF);
    hn = (char *) malloc(hnlen);
    if (NULL == hn) {
      free(dn);
      return NULL;
    }
    gethostname(hn, SML_BUF);

    /* Make FQDN */
    if (!strchr(hn, '.') && *dn) {
      snprintf(hn, hnlen, "%s.%s", hn, dn);
      if (!memchr(hn, 0, hnlen)) {
        free(hn);
        free(dn);
        return NULL;
      }
    }

    free(dn);

    /* Return whatever we ended up with. */
    return hn;
#endif
  }

/*********************************************************************
**
** FUNCTION: setupGetNetworkDomain
** Replaced: GetNetworkDomain
**
** DESCRIPTION: returns the network Domain
**
**
** INPUTS: none
**
** RETURN:
**   network domain or null if unable to get host name
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**  need to free memory by calling setupFree
**********************************************************************/
NSAPI_EXTERN char * setupGetNetworkDomain(void)
{
#ifdef XP_WIN32
  char szRegKey[MAX_PATH];
  unsigned char szTempDomain[MAX_PATH];
  char *szDomain = NULL;
  HKEY hKey;
  snprintf(szRegKey, sizeof(szRegKey), REGSTR_PATH_SERVICES "\\Tcpip\\Parameters");
  if (!memchr(szRegKey, 0, sizeof(szRegKey))) return NULL; /*overflow*/

  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
  {
    unsigned long cbLen;
    cbLen = sizeof(szTempDomain);
    if (RegQueryValueEx(hKey, "Domain", 0, NULL, szTempDomain, &cbLen) != ERROR_SUCCESS)
      szTempDomain[0] = '\0';
    RegCloseKey(hKey);
  }
  else
  {
    szTempDomain[0] = '\0';
  }
  szDomain = strdup((const char *)szTempDomain);
  return szDomain;
#else
  FILE *f = fopen("/etc/resolv.conf", "r");
  char line[SML_BUF];
  char *domain;
  char *dm;

  domain = (char *) malloc(SML_BUF);
  /* See if there's a domain entry in their resolver configuration */
  if (f)
  {
    while (fgets(line, sizeof(line), f))
    {
      if (!strncasecmp(line, "domain ", 7))
      {
        sscanf(&line[7], "%s", domain);
        dm = (domain[0] == '.' ? &domain[1] : domain);
        return dm;
      }
    }
    fclose(f);
  }

#ifndef NO_DOMAINNAME
  /* No domain found. Try getdomainname. */
#ifdef SOLARIS
  sysinfo(SI_SRPC_DOMAIN, domain, SML_BUF);
#else
  getdomainname(domain, SML_BUF);
#endif
  dm = (domain[0] == '.' ? &domain[1] : domain);
#endif

  return dm;

#endif

  }
  /*********************************************************************
  **
  ** FUNCTION:
  ** Replaced:
  **
  ** DESCRIPTION:
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN unsigned int setupGetRandomNetworkPort(void)
  {
#ifdef XP_WIN32
    unsigned int nPort;
    WSADATA wsaData;
    SOCKADDR_IN sin;
    SOCKET s;
    srand(time(NULL));

    if (WSAStartup(MAKEWORD(1,1), &wsaData) != 0)
    {
      nPort = MINPORT + rand() % (MAXPORT - MINPORT);
    }
    else
    {
      for (;;)
      {
        nPort = MINPORT + rand() % (MAXPORT - MINPORT);
        if ((s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET)
        {
          continue;
        }
        else
        {
          sin.sin_family = AF_INET;
          sin.sin_addr.s_addr =  htonl(INADDR_ANY);
          sin.sin_port = htons((u_short)nPort);
          if (bind(s, (LPSOCKADDR)&sin, sizeof (sin)) == 0)
          {
            if (WSAGetLastError() != WSAEADDRINUSE)
              continue;
          }
          closesocket(s);
        }
      }

      WSACleanup();
    }

    return nPort;
#else
    int port;

    srand(time(NULL));
    while (1)
    {
      port = MINPORT + rand() % (MAXPORT-MINPORT);

      if (setupIsValidNetworkPort(port) == True)
        return port;
    }
    return port;
#endif
  }

  /*********************************************************************
  **
  ** FUNCTION:
  ** Replaced:
  **
  ** DESCRIPTION:
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN int setupIsValidNetworkPort(unsigned int iPort)
  {
#ifdef XP_WIN32
    int  iPortValid = TRUE;
    WSADATA wsaData;
    SOCKADDR_IN sin;
    SOCKET s;


    if ((iPort < 1) || (iPort > MAXPORT))
    {
      iPortValid = FALSE;
      return iPortValid;
    }
    if (WSAStartup(MAKEWORD(1,1), &wsaData) == 0)
    {
      if ((s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) != INVALID_SOCKET)
      {
        sin.sin_family = AF_INET;
        sin.sin_addr.s_addr = htonl(INADDR_ANY);
        sin.sin_port = htons((u_short)iPort);
        if ((bind(s, (LPSOCKADDR)&sin, sizeof(sin)) == SOCKET_ERROR) && (WSAGetLastError() == WSAEADDRINUSE))
        {
          iPortValid = FALSE;
        }
        closesocket(s);
      }

      WSACleanup();
    }

    return iPortValid;
#else
    struct sockaddr_in sa_server;
    int sd, ret;
    int one = 1;

    if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      return False;
    /* atulb: 6/17/98 - bug#135631 : commenting out this as it is unnecessary and
       allowed multiple binds to the same port on the localhost
       if ((setsockopt(sd, SOL_SOCKET, SO_REUSEADDR,
                       (const char *) &one, sizeof(one))) == -1)
          return False;
    */
    memset((char *) &sa_server, 0, sizeof(sa_server));
    sa_server.sin_family = AF_INET;
    sa_server.sin_addr.s_addr = htonl(INADDR_ANY);
    sa_server.sin_port = htons(iPort);
    ret = bind(sd, (struct sockaddr *) & sa_server, sizeof(sa_server));
    close(sd);
    return (ret == -1) ? False : True;
#endif
  }
  /*********************************************************************
  **
  ** FUNCTION: setupGetFileString
  ** Replaced: GetProductInfoString
  **
  ** DESCRIPTION:
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN char *setupGetFileString(const char* pszKey, const char* sep, const char* pszFileName)
  {
    FILE *fp;
    char *buf;
    size_t    bufsiz = MAX_PATH;
    char *string = NULL;
    char *pszValue = NULL;

    if ((fp = fopen(pszFileName, "r")) != NULL)
    {
        char *szKey;
        int szKeyLen = strlen(pszKey);
        szKey = (char *)malloc(szKeyLen + 1);
        buf = (char *) malloc(bufsiz);
        if ((szKey == NULL) || (buf == NULL)) return NULL;
        while (fgets(buf, bufsiz, fp) != NULL)
        {
          while ( strlen(buf) > bufsiz - 1 )
          {
            /* line longer than bufsiz */
            buf = (char *)realloc( buf, bufsiz * 2 );
            if (buf == NULL) return NULL;
            if (fgets( buf + bufsiz - 1, bufsiz + 1, fp ) == NULL)
            {
              return FALSE;
            }
            bufsiz *= 2;
          }
          strncpy(szKey, buf, szKeyLen);
          szKey[szKeyLen] = '\0';
          if (strlen(szKey) >= strlen(pszKey))
          {
            szKey[strlen(pszKey)] = '\0';
            if (strcmp(szKey, pszKey) == 0)
            {
              pszValue = buf;
              pszValue = pszValue+(strlen(pszKey));
              if (sep)
              {
                char *pszToken;
                if (strlen(sep) == 1)
                {
                  if (pszValue[0] == *(sep))
                  {
                    pszValue = pszValue + 1;
                  }
                }
                else
                {
                  pszToken = strtok(pszValue, sep);
                  if (pszToken != NULL)
                  {
                    pszValue = pszToken;
                  }
                }
              }
              string = strdup(pszValue);
              break;
            }
          }
        }
        if (szKey) setupFree(szKey);
        if (buf) setupFree(buf);
        fclose(fp);
    }

    //remove leading and trailing whitespace
    if (string)
    {
      char * s;
      char * r;

      r = s = string;
      while (s && i18n_isspace( *s ))
      {
        s++;
      }

      if (s)  //now for the trailing white space
      {
        int i;
        i = strlen(s);
        i--;
        while ((i >= 0) && (i18n_isspace(s[i])))
        {
           i--;
        }
        s[++i] = '\0';
      }
      string = strdup(s);
      setupFree(r);
    }

    return string;
  }
  /*********************************************************************
  **
  ** FUNCTION: setupGetFileInt
  ** Replaced: GetProductInfoInt
  **
  ** DESCRIPTION:
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/
  NSAPI_EXTERN int setupGetFileInt(const char* pszKey, const char* sep, const char* pszFileName)
  {
    char *szNumber;

    szNumber = setupGetFileString(pszKey, sep, pszFileName);
    if (szNumber == NULL)
      return -1;
    setupFree(szNumber);
    return atoi(szNumber);
  }



  /*********************************************************************
  **
  ** FUNCTION:
  ** Replaced:
  **
  ** DESCRIPTION:
  **
  **
  ** INPUTS:
  **
  ** RETURN:
  **   TRUE or FALSE
  **
  ** SIDE EFFECTS:
  **      none
  ** RESTRICTIONS:
  **      None
  ** MEMORY:
  **********************************************************************/

  NSAPI_EXTERN char *
  setupGetInfString(const char* pszSection, const char* pszKey, const char* pszDefault, const char* pszFileName)
  {
    const char *string;
    char *ReturnString;
    NVPair *inf;
    NVPair *section;
    inf = new NVPair;

    inf->setFormat(1);
    inf->read(pszFileName);
    if (inf->isEmpty())
    {
      return pszDefault ? strdup(pszDefault) : NULL;
    }
    if (strcmp(pszSection, SETUP_INF_GENERAL_SECTION) != 0)
    {
      section = inf->getSection(pszSection);
    }
    else
    {
      section = inf;
    }
    if (section == NULL)
    {
      return pszDefault ? strdup(pszDefault) : NULL;
    }
    string = section->get(pszKey);


    if (string != NULL)
      ReturnString = strdup(string);
    else
      ReturnString = pszDefault ? strdup(pszDefault) : NULL;


    delete inf;

    return ReturnString;
  }
/*********************************************************************
**
** FUNCTION:
** Replaced:
**
** DESCRIPTION:
**
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/

NSAPI_EXTERN int setupGetInfInt(const char* pszSection, const char* pszKey, int iDefault, const char* pszFileName)
{
   const char *string;
   NVPair inf;
   NVPair *section;
   int iRet = iDefault;

   inf.setFormat(1);
   inf.read(pszFileName);

   if (inf.isEmpty())
   {
      return iRet;
   }

   if (strcasecmp(pszSection, SETUP_INF_GENERAL_SECTION) != 0)
   {
      section = inf.getSection(pszSection);
   }
   else
   {
      section = &inf;
   }

   if (section == NULL)
   {
      return iRet;
   }

   if (string = section->get(pszKey))
      iRet = atoi(string);

   return iRet;
}


/*********************************************************************
**
** FUNCTION:
** Replaced:
**
** DESCRIPTION:
**
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/

NSAPI_EXTERN int setupWriteInfString(const char* pszSection, const char* pszKey, const char* pszValue, const char* pszFileName)
{
#ifndef XP_WIN32
   NVPair inf;
   NVPair *section;
   int iRet = True;

   inf.setFormat(1);
   inf.read(pszFileName);

   section = &inf;
   if (strcasecmp(pszSection, SETUP_INF_GENERAL_SECTION) != 0)
   {
      if ((section = inf.getSection(pszSection)) == NULL)
         section = inf.createSection(pszSection);
   }

   if (section == NULL)
   {
      return False;
   }

   section->set(pszKey, pszValue);

   inf.setFormat(1);
   inf.write(pszFileName);

   return iRet;
#else
   return WritePrivateProfileString(pszSection, pszKey, pszValue, pszFileName);
#endif
}

/*********************************************************************
**
** FUNCTION:
** Replaced:
**
** DESCRIPTION:
**
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/
NSAPI_EXTERN int setupWriteInfInt(const char* pszSection, const char* pszKey, int i, const char* pszFileName)
{
  char buf[1024];
  snprintf(buf, sizeof(buf), "%d", i);
  return setupWriteInfString(pszSection, pszKey, buf, pszFileName);
}


/*********************************************************************
**
** FUNCTION: returns true if running under Windows NT
** Replaced:
**
** DESCRIPTION:
**
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/
NSAPI_EXTERN int setupIsWindowsNT(void)
{
    #ifndef XP_WIN32
    return FALSE;
    #else
    BOOL bReturn = FALSE;
    OSVERSIONINFO osVersionInfo;

    osVersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    if(GetVersionEx(&osVersionInfo))
    {
        bReturn = (osVersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
    }
    return(bReturn);
    #endif
}


/*********************************************************************
**
** FUNCTION: setupExecProgram
**
** DESCRIPTION:
**
**
** INPUTS:
**   program   -   program to launch
**   where     -   directory name for starting program from
**
** RETURN:
**   return value from program.
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/
NSAPI_EXTERN unsigned int setupExecProgram(const char *program, const char *where)
{
   char curdir[MAX_PATH];
   unsigned int ret;
#ifdef XP_WIN32
   char szWhere[MAX_PATH];
   char szProgram[MAX_PATH];
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
   DWORD dwCreationFlags;
#endif

   if (!program)
   {
      setupLogMessage("info", "setup", "setupExecProgram called with no program");
      return 0;
   }
   if (!setupFileExists(program))
   {
      setupLogMessage("info", "setup", "setupExecProgram called with invalid program %s", program);
   }
   getcwd(curdir, sizeof(curdir));

   if (where)
   {
      chdir(where);
   }
#ifndef XP_WIN32
   ret = system(program);
#else
   snprintf(szProgram, sizeof(szProgram), "%s", program);
   if (!memchr(szProgram, 0, sizeof(szProgram))) {
      setupLogMessage("info", "setup", "Too long program %s", program);
      return 0;
   }
   snprintf(szWhere, sizeof(szWhere), "%s", where);
   if (!memchr(szWhere, 0, sizeof(szWhere))) {
      setupLogMessage("info", "setup", "Too long arg %s", where);
      return 0;
   }
   memset(&si,0,sizeof(si));
   si.cb = sizeof(si);
   GetStartupInfo(&si);
   si.wShowWindow = SW_SHOWDEFAULT;
   dwCreationFlags = NORMAL_PRIORITY_CLASS | DETACHED_PROCESS;
   ret = CreateProcess(NULL, szProgram, NULL, NULL, FALSE, dwCreationFlags, NULL, szWhere, &si, &pi);
#endif

   if (where)
   {
      chdir(curdir);
   }

   return ret;
}


/*********************************************************************
**
** FUNCTION:
** Replaced:
**
** DESCRIPTION:
**
**
** INPUTS:
**
** RETURN:
**   TRUE or FALSE
**
** SIDE EFFECTS:
**      none
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************/


NSAPI_EXTERN void
setupGetGmtime(char *buf, int buflen)
{
   time_t curtime;
   struct tm *tm_p;

   curtime = time(NULL);
   tm_p = gmtime(&curtime);

   strftime(buf, buflen, "%Y%m%d%H%M%SZ", tm_p);

   return;
}


NSAPI_EXTERN char *
setupLocalToUtf8(const char *src)
{
   return localToUTF8(src);
}

NSAPI_EXTERN char *
setupUTF8ToLocal(const char *utf8)
{
   return UTF8ToLocal(utf8);
}

NSAPI_EXTERN int setupIsAsciiBuffer(const char *buf)
{
   return isAsciiBuffer(buf);
}

NSAPI_EXTERN int setupBase64_decode( char *src, unsigned char *dst )
{
   return base64_decode(src, dst );
}

NSAPI_EXTERN int setupBase64_Encode( unsigned char *src, char *dst, int srclen, int lenused )
{
   return base64_encode( src, dst, srclen, lenused );
}

NSAPI_EXTERN char *setupSHA1_pw_enc( char *pwd )
{
   return sha1_pw_enc( pwd );
}

NSAPI_EXTERN int setupSHA1_pw_cmp( char *userpwd, char *dbpwd )
{
   return sha1_pw_cmp(userpwd, dbpwd );
}


} /* End extern "C" */

/*********************************************************************
**
** CLASS: InstallLog
**
*********************************************************************/

const char *SETUP_LOG_FATAL   = "Fatal";
const char *SETUP_LOG_START   = "Start";
const char *SETUP_LOG_SUCCESS = "Success";
const char *SETUP_LOG_WARN    = "Warning";
const char *SETUP_LOG_INFO    = "Info";


// backward compatible with ux-utils.cpp
const char *FATAL    = "Fatal";
const char *START    = "Start";
const char *SUCCESS  = "Success";
const char *WARN     = "Warning";
const char *INFO     = "Info";

char * InstallLog::_logFile = NULL;

/*********************************************************************
**
** FUNCTION:    InstallLog
** DESCRIPTION:
**
** INPUTS:      None
** OUTPUTS:
** RETURN:
**   Current Directory
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/
InstallLog::InstallLog (const char *fileName)
   :_valid(False)
{
   FILE *fp;

   if (fileName == NULL || _logFile != NULL || (fp = fopen(fileName, "a")) == NULL)
   {
      return;
   }
   else
   {
      _logFile = strdup(fileName);
      fputs("\n", fp);
      fclose(fp);
      _valid = True;
   }
}

InstallLog::~InstallLog()
{
   if (_logFile) free(_logFile);
   _logFile = NULL;
}

Bool
InstallLog::isValid() const
{
   return (_valid);
}

void
InstallLog::logTime(const char *who)
{
  char buf[SML_BUF];
  time_t clock = time(NULL);
  strncpy(buf, ctime(&clock), SML_BUF);
  buf[strlen(buf)-1] = 0;
  logMessage(INFO, who, "Log starts %s", buf);
}

void
InstallLog::logMessage (const char *level, const char *who, const char *msg, ...)
{
   char buf[BIG_BUF];
   va_list arg;
   FILE *fp;
   struct tm *newtime;
   time_t ltime;
   char *s;
   int buflen;

   if (_logFile == NULL || (fp = fopen(_logFile, "a")) == NULL)
   {
      return;
   }


   time( &ltime );
   newtime = localtime( &ltime );
   strftime(buf, BIG_BUF, "[%y/%m/%d:%H:%M:%S] - ", newtime);

   s = buf + strlen(buf);
   buflen = sizeof(buf) - (s - buf);

   if (NULL == who) {
      if (NULL == level) {
         ;
      } else {
         snprintf(s, buflen, "%s ", level);
         s[buflen-1] = '\0'; /* in case overflowed */
      }
   } else {
      if (NULL == level) {
         snprintf(s, buflen, "[%s] ", who);
         s[buflen-1] = '\0'; /* in case overflowed */
      } else {
         snprintf(s, buflen, "[%s] %s ", who, level);
         s[buflen-1] = '\0'; /* in case overflowed */
      }
   }

   s = buf + strlen(buf);
   buflen = sizeof(buf) - (s - buf);
   if (buflen > 0) {
      va_start (arg, msg);
      vsnprintf(s, buflen, msg, arg);
      va_end(arg);
   }

   fputs(buf,fp);
   fputs("\n",fp);

   fclose(fp);

   return;
}

void
InstallLog::log (const char *fileName, const char *msg)
{
   if (fileName)
   {
      FILE *fp;
      fp = fopen(fileName, "a");

      if (fp)
      {
         char buf[BIG_BUF];
         struct tm *newtime;
         time_t ltime;
         int buflen;

         time( &ltime );
         newtime = localtime( &ltime );
         strftime(buf, BIG_BUF, "[%y/%m/%d:%H:%M:%S] - ", newtime);
         buflen = sizeof(buf) - strlen(buf);

         strncat(buf, msg, buflen);
         buf[sizeof(buf) - 1] = '\0'; /* in case overflowed */

         fputs(buf,fp);
         fputs("\n",fp);

         fclose(fp);
      }
   }
}

/*********************************************************************
**
** FUNCTION:    setupFindNameInList
** DESCRIPTION: Check if given name is already on the list.  
**
** INPUTS:      listValue:   name/value list
**              name:        name to look for
** RETURN:
**              True:        name is in the list
**              False:       name is not in the list
**********************************************************************
*/

Bool setupFindNameInList(const char *listValue, const char *name)
{
  char *tok;
  char *list = NULL;
  char *item = NULL;
  Bool bRet = False;

  if (listValue == NULL)
    return False;

  list = strdup(listValue);

  for (tok = strtok(list,", "); tok; tok = strtok(NULL,", "))
  {
    char *s;

    item = strdup(tok);
    if (s = strchr(item,'/'))
      *s = '\0';

    /*
     *  Name maches item itself or name part of item
     */
    if (strcasecmp(item, name) == 0 || strcasecmp(tok, name) == 0)
    {
      bRet = True;
      free (item);
      break;
    }

    free (item);
  }
  
  free (list);

  return bRet;
}

/*********************************************************************
**
** FUNCTION:    setupAddNameToList
** DESCRIPTION:  Add name to a list.  If list is not there, it will create
**               new one. If the name is already on the list, it will return
**               a copy of the list. 
** INPUTS:      listValue:   name/value list
**              name:        name to add to the list
** RETURN:      new name/value list
**********************************************************************
*/

char * setupAddNameToList(const char *listValue, const char *name)
{
  if (listValue == NULL || *listValue == '\0')
  {
    return strdup(name);
  }

  if (setupFindNameInList(listValue, name) == True)
  {
    return strdup(listValue);
  }
  else
  {
    char *buffer;
    int buflen = strlen(listValue) + strlen(name) + 1 + 1;
    buffer = (char *) malloc(buflen);
    if (NULL == buffer)
        return NULL;
    snprintf(buffer, buflen, "%s,%s", listValue, name);
    return buffer;
  }
}

/*********************************************************************
**
** FUNCTION:      setupGetDiskFreeSpace
** DESCRIPTION:   get available disk space in Kilobyte
**
** INPUTS:        Path name
** OUTPUTS:
** RETURN:        available disk space in kilobyte
** SIDE EFFECTS:
**      None
** RESTRICTIONS:
**      None
** MEMORY:
**********************************************************************
*/

unsigned long setupGetDiskFreeSpace(const char *path)
{
#ifdef XP_WIN32
  DWORD dwSectorsPerCluster;
  DWORD dwBytesPerSector;
  DWORD dwFreeClusters;
  DWORD dwTotalClusters;

  if (GetDiskFreeSpace(path, &dwSectorsPerCluster, &dwBytesPerSector, &dwFreeClusters, &dwTotalClusters))
  {
     long long clusterSize;

     clusterSize = dwSectorsPerCluster * dwBytesPerSector;

     return (unsigned long)((clusterSize / 1024.0) * dwFreeClusters);
  }

  return 0;
#else
  struct statvfs buf;

  statvfs(path, &buf);

  if (buf.f_frsize == 0)
    return (unsigned long)(((unsigned long long)buf.f_bavail / 1024.0) * buf.f_bsize);
  else
    return (unsigned long)(((unsigned long long)buf.f_bavail / 1024.0) * buf.f_frsize);
#endif
}

/*********************************************************************
**
** FUNCTION:      setupFileUpdateLines
** DESCRIPTION:   replace multiple lines to new lines in existing text file.
**
** INPUTS:        
**   oldLines -- existing lines to match.  
**   newLines -- replacement lines for each oldLines    
** OUTPUTS:
**   if successful, the file will be replaced with new one. 
** RETURN:        
**   0     -- success
**   1     -- file can't open for read
**   2     -- can't write the file.
**   3     -- out of memory
** Note:
**   setupReplaceLines does case insensitive match between each line from file
**   and oldLines.  When matching, string length is based on oldLines. 
**   Thus caller can pass "partial" line as oldLines and replaced it with
**   newLines.  oldLines and newLines should be in sync. Last item in oldLines
**   should be set to NULL.
**********************************************************************
*/


int setupFileUpdateLines(char *oldLines[], char *newLines[], char *fileName)
{
  char line[1024];
  struct stat statbuf;
  long size;
  FILE *fp;
  char *buffer;
  int origbuflen;
  int buflen;
  int *updateList;
  int n;
  int extraSize;
  int i;

  n = 0;
  extraSize = 0;
  while (newLines[n] != NULL)
  {
    extraSize += strlen(newLines[n]);
    n ++;
  }

  updateList = (int *) malloc(n);
  if (updateList == NULL)
      return 3;
  for (i = 0; i < n; i++)
  {
    updateList[i] = 0;
  }

  if (stat(fileName, &statbuf) != 0)
  {
    buflen = extraSize + 1;
    buffer = (char *) malloc(buflen);
    if (buffer == NULL)
      return 3;
    buffer[0] = '\0';
  }
  else
  {
    size = statbuf.st_size;
  
    if ((fp = fopen(fileName, "r")) == NULL)
      return 1;       

    /*
     *  get replacement count
     */                      

    origbuflen = buflen = size + extraSize + 1;
    buffer = (char *) malloc(buflen);
    if (buffer == NULL)
      return 3;

    buffer[0] = '\0';  

    /*
     *  replace necessary lines and save them to memory
     */
    while(fgets(line, sizeof(line), fp))
    {
      for (i = 0; i < n; i ++)
      {
        if (strncasecmp(line, oldLines[i], strlen(oldLines[i])) == 0)
        {
          strncat(buffer, newLines[i], buflen);
          buflen -= strlen(newLines[i]);
          strncat(buffer, "\n", buflen);
          buflen -= 1;
          if (buflen <= 0) {
            buffer[origbuflen - 1] = '\0';
            goto error_out; /* overflow: write at least what we have */
          }
          updateList[i] = 1;
        }
        else
        {
          strncat(buffer, line, buflen);
          buflen -= strlen(line);
          strncat(buffer, "\n", buflen);
          buflen -= 1;
          if (buflen <= 0) {
            buffer[origbuflen - 1] = '\0';
            goto error_out; /* overflow: write at least what we have */
          }
        }
      }
    }
    fclose(fp);
  }

  for (i = 0; (i < n) && (updateList[i] == 0); i++)
  {
    strncat(buffer, newLines[i], buflen);
    buflen -= strlen(newLines[i]);
    strncat(buffer, "\n", buflen);
    buflen -= 1;
    if (buflen <= 0) {
      buffer[origbuflen - 1] = '\0';
      goto error_out; /* overflow: write at least what we have */
    }
    updateList[i] = 1;
  }
error_out:
  /*
   *  write from memory to file
   */
  if ((fp = fopen(fileName, "w")) == NULL)
    return 2;

  fwrite(buffer, sizeof(char), strlen(buffer), fp);

  free(buffer);

  fclose(fp);

  return 0;
}
