/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** NAME
**   dl-common.h
**
** DESCRIPTION
**   Common dialog routines
**
** NOTES:
**
** HISTORY
**
** $Log: dl-common.h,v $
** Revision 1.1.1.1  2005/07/29 22:16:30  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.1  1999/03/04 23:27:00  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.10.4.1  1998/10/29 03:13:34  cxue
** move dialog resource from dl-common.cc to uxres_data.cc
**
** Revision 1.1.2.10  1998/07/16 17:12:21  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.9  1998/06/24 17:41:13  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.8  1998/05/29 16:55:12  pvo
** 117356 - component selection accepts any selection
**
** Revision 1.1.2.7  1998/02/26 00:13:57  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.6  1998/02/24 00:23:06  pvo
** Support for topology changes.
**
** Revision 1.1.2.5  1998/02/17 23:36:49  pvo
** Fixed problem with defaults for Yes/No answer.
**
** Revision 1.1.2.4  1998/02/15 22:06:38  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.3  1997/12/12 21:54:15  pvo
** Various bug fixes.
**
** Revision 1.1.2.2  1997/11/26 02:02:52  pvo
** Restructure files and add Download support.
**
**
***********************************************************************/

#ifndef NS_DLCOMMON_H
#define NS_DLCOMMON_H

#include "product.h"
#include "dialog.h"

extern Bool yesNoDefaultNo(const char *answer);
extern Bool yesNoDefaultYes(const char *answer);

extern DialogAction welcomeSetup(Dialog *me);
extern DialogAction askSelectionNext (Dialog *me);
extern DialogAction askModuleNext(Dialog *me);
extern DialogAction askLdapInfoSetup(Dialog *me);
extern DialogAction askLdapInfoNext(Dialog *me);
extern DialogAction askLdapUserSetup(Dialog *me);
extern DialogAction askLdapUserNext(Dialog *me);
extern DialogAction askUserDirectoryInfoSetup(Dialog *me);
extern DialogAction askUserDirectoryInfoNext(Dialog *me);
extern DialogAction askUserDirectoryAdminSetup(Dialog *me);
extern DialogAction askUserDirectoryAdminNext(Dialog *me);

extern DialogAction verifyLdapInfo(Dialog *me, const char *ldapURL, const char *ldapLabel);
extern DialogAction getLdapPassword(Dialog *me, const char *ldapURL, const char *ldapUser);

Dialog *createSelectionDialog(Components *, const char *prompt, DLG_CALLBACK, const char *selectionText, const char *whatProduct=NULL);
extern void dialogSetup (Dialog *me, const char *which);
int getSelectedComponent(Dialog *me, Components *components, Bool express);
void setupComponentDialog(Dialog *me, Components *components);

extern DialogInput askLdapInfo;

extern DialogInput askLdapUser;

extern DialogInput askUserDirectoryInfo;

extern DialogInput askUserDirectoryAdmin;

extern DialogInput askInstallDN;

#endif
