/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** NAME
**   dl-inst.cc
**
** DESCRIPTION
**   Common dialog routines for Installer
**
** NOTES:
**
** HISTORY
**
** $Log: dl-inst.cc,v $
** Revision 1.1.1.1  2005/07/29 22:16:31  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.6  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.5  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.4  2005/03/28 21:14:47  nhosoi
** Fixed the Server selection page; 2. Management Console selection was missing.
**
** Revision 1.3  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/03 22:18:51  nhosoi
** 1) extracted hardcoded brandnames from error messages
** 2) <Brand> Console is changed to <Brand> Management Console (uxres_data.cc)
** modified files: dl-inst.cc,installer.cc,uxres.cc,uxres.h,uxres_data.cc
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.8.4.2  2003/08/26 19:30:23  djani
** Bug 605982: Now we check if the computer name is 7-bit clean. If not, display an error message and reprompt the user.
**
** Revision 1.1.1.8.4.1  2002/04/13 00:45:39  djani
** Bug 389570: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.8  2001/09/20 21:36:36  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.7  2000/03/06 00:59:01  peterz
** change "Netscape" to "iPlanet"
**
** Revision 1.1.1.6  1999/12/04 00:34:57  cxue
** SecurityCheck and RequireDomain support in setup.inf
**
** Revision 1.1.1.5  1999/10/13 00:19:24  cxue
** chnage <serverroot> and go forward
**
** Revision 1.1.1.4  1999/10/01 20:56:05  cxue
** Warning when setup can't find domain name
**
** Revision 1.1.1.3  1999/09/27 20:18:38  cxue
** check diskspace after selection is done
**
** Revision 1.1.1.2  1999/09/17 01:48:45  cxue
** fix long lasting cross-dependence bug
**
** Revision 1.1.1.1  1999/03/04 23:27:01  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.32.2.9  1999/02/04 02:24:43  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.32.2.8  1998/11/06 20:29:44  cxue
** compilation
**
** Revision 1.1.2.32.2.7  1998/10/29 03:13:34  cxue
** move dialog resource from dl-common.cc to uxres_data.cc
**
** Revision 1.1.2.32.2.6  1998/10/28 23:47:44  cxue
** move dialog resource from dl-inst to uxres_data
**
** Revision 1.1.2.32.2.5  1998/10/28 17:55:50  cxue
** support userdefined default setting
**
** Revision 1.1.2.32.2.4  1998/10/15 22:22:05  cxue
** resource framework for custom ui
**
** Revision 1.1.2.32.2.3  1998/10/05 20:57:43  cxue
** test NULL pointer
**
** Revision 1.1.2.32.2.2  1998/10/02 18:05:36  cxue
** Support mode=express|typical|custom|all modes in [general] section. New hidden attribute for Dialog class
**
** Revision 1.1.2.32.2.1  1998/10/01 02:38:37  pvo
** 323714 - Fixed trailing / problem when specifying server-root
**
** Revision 1.1.2.32  1998/08/24 18:27:12  pvo
** 315637 - Fixed typo in legal screen
**
** Revision 1.1.2.31  1998/08/20 18:07:11  cxue
** Reinstall won't ask for ldapinfo
**
** Revision 1.1.2.30  1998/07/31 02:56:58  pvo
** Fix version check logic
**
** Revision 1.1.2.29  1998/07/17 22:08:06  cxue
** Bug 301470: change unix installer not to ask host name in express mode. Finally NT and Unix have something in common.
**
** Revision 1.1.2.28  1998/07/16 17:12:22  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.27  1998/07/02 03:30:32  pvo
** Some minor name changes
**
** Revision 1.1.2.26  1998/06/19 03:08:57  pvo
** 154173 - change MC Console to Netscape Console
**
** Revision 1.1.2.25  1998/05/29 16:55:14  pvo
** 117356 - component selection accepts any selection
**
** Revision 1.1.2.24  1998/05/13 04:39:30  pvo
** 110487 - Support U/G separation
**
** Revision 1.1.2.23  1998/05/01 23:24:41  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.22  1998/04/01 02:42:29  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.21  1998/03/13 05:00:36  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.20  1998/03/10 02:49:14  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.19  1998/03/09 22:09:36  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.18  1998/03/02 23:59:26  pvo
** 106912 - Pickup the right defaults in dialogs.
** 105320 - Improve logging of silent installation.
**
** Revision 1.1.2.17  1998/02/26 00:13:58  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.16  1998/02/24 00:23:06  pvo
** Support for topology changes.
**
** Revision 1.1.2.15  1998/02/17 23:36:50  pvo
** Fixed problem with defaults for Yes/No answer.
**
** Revision 1.1.2.14  1998/02/15 22:06:39  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.13  1998/01/22 04:51:27  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.12  1998/01/09 03:27:28  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.11  1997/12/12 21:54:16  pvo
** Various bug fixes.
**
** Revision 1.1.2.10  1997/12/04 00:03:12  pvo
** Prepare support for DS installation.
**
** Revision 1.1.2.9  1997/12/02 04:28:27  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.8  1997/11/26 02:02:54  pvo
** Restructure files and add Download support.
**
**
***********************************************************************/

extern "C"
{
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
}

#include "ldapu.h"
#include "ux-util.h"
#include "dialog.h"
#include "dl-common.h"
#include "dl-inst.h"
#include "installer.h"

#include "uxres.h"

/*
** Forward References
**/
int
checkTarFile (Dialog *, const char *program);

DialogAction welcomeNext (Dialog *me);
DialogAction askLicenseSetup (Dialog *me);
DialogAction askLicenseNext (Dialog *me);
DialogAction askClientOrServerSetup(Dialog *me);
DialogAction askClientOrServerNext(Dialog *me);
DialogAction askInstallTypeSetup(Dialog *me);
DialogAction askInstallTypeNext(Dialog *me);
DialogAction askServerRootSetup(Dialog *me);
DialogAction askClientRootSetup(Dialog *me);
DialogAction askServerRootNext(Dialog *me);
DialogAction askHostNameSetup(Dialog *me);
DialogAction askHostNameNext(Dialog *me);
DialogAction askSSUserSetup(Dialog *me);
DialogAction askSSUserNext(Dialog *me);

DialogAction askSelectionNext (Dialog *me);
DialogAction removeInstallInfoSetup(Dialog *me);
DialogAction removeInstallInfoNext(Dialog *me);
DialogAction finishSelectionSetup(Dialog *me);
DialogAction finishSelectionNext(Dialog *me);


const char *INSTALL_WHICH = "InstallWhich";
const char *INSTALL_SERVERS = "1";
const char *INSTALL_CLIENTS = "2";





/////////////////////////////////////////////////////////////////////////
/// Welcome 
/////////////////////////////////////////////////////////////////////////

DialogYesNo welcome(
NULL,
NULL,
"Yes",
welcomeSetup,
welcomeNext
);

DialogAction welcomeNext (Dialog *me)
{
   const char *buf = me->input();
   DialogAction rc;

   if (buf[0] == 0)
   {
      buf = me->defaultAns();
   }
   
   rc = yesNoDefaultYes(buf) ? DIALOG_NEXT:DIALOG_TERMINATE;
   if (rc == DIALOG_NEXT && me->isVisited() == False)
   {
      me->setDefaultAns(buf);
      askLicense.registerDialogNext(me->manager());
      me->visited();
   }
   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// License 
/////////////////////////////////////////////////////////////////////////

DialogYesNo askLicense(
NULL,
NULL,
"No",

askLicenseSetup,
askLicenseNext
);

DialogAction askLicenseSetup(Dialog *me)
{
   Installer *installer = (Installer *) me->manager()->parent();
   const char *s;

   me->setText(getResource(RES_DIALOG_LICENSE_TEXT));
   me->setPrompt(getResource(RES_DIALOG_LICENSE_PROMPT));

   // skip license dialog if someone knows how to set it
   s = installer->getUserDefault()->get("skipLicenseDialog");
   if (s && !strcasecmp(s, "true"))
   {
      me->setDefaultAns("Yes");
      me->hidden();
   }

   return DIALOG_SAME;
}

DialogAction askLicenseNext (Dialog *me)
{
  const char *buf = me->input();
  DialogAction rc;
  Installer *installer = (Installer *) me->manager()->parent();
  InstallInfo *installInfo = me->manager()->getDefaultScript();

  if (buf[0] == 0)
  {
     buf = me->defaultAns();
  }

  rc = yesNoDefaultNo(buf) ? DIALOG_TERMINATE:DIALOG_NEXT;

  if (rc == DIALOG_NEXT && me->isVisited() == False)
  {
    char *domain;
    // Alert if no domain name on this machine.
    if (installer->needDomainName())
    {
      domain = setupGetNetworkDomain();
      if ( domain == NULL || *domain == '\0')
      {
        DialogYesNo *msgDialog = new DialogYesNo(
"Warning: Installation cannot determine the Domain Name \n"
"for this Host. Your Network Settings may not be correct, \n"
"or your host may be on a DHCP network. If you are using \n"
"TCP/IP your Domain Name must be filled in. \n\n",
"No");

        rc = msgDialog->execute();
        if (rc != DIALOG_NEXT)
        {
          if (rc == DIALOG_SAME)
            return DIALOG_TERMINATE;
          else
            return rc;
        }
      }
    }

    if (installer->hasClients() == False)
    {
       askInstallType.registerDialogNext(me->manager());
       installInfo->set(INSTALL_WHICH, INSTALL_SERVERS);
    }
    else if (installer->allClients() == True)
    {
       installInfo->set(INSTALL_WHICH, INSTALL_CLIENTS);
       askClientRoot.registerDialogNext(me->manager());
    }
    else
    {
       installInfo->set(INSTALL_WHICH, INSTALL_SERVERS);
       askClientOrServer.registerDialogNext(me->manager());
    }
    me->visited();
    me->setDefaultAns(buf);
  }

  return rc;
}


/////////////////////////////////////////////////////////////////////////
/// Selection Status 
/////////////////////////////////////////////////////////////////////////
DialogYesNo finishSelection(
"this is testing",
"Hit Enter key to go back",
"No",
finishSelectionSetup,
finishSelectionNext
);

DialogAction finishSelectionSetup (Dialog *me)
{
   NSString msg;
   Installer *installer = (Installer *) me->manager()->parent();

   // global check on component dependencies
   msg = installer->checkComponentsDependency(installer->getInstallableComponents());
   if (msg != (char *) NULL)
   {
      me->setText (msg);
   }    
   else if (installer->checkDiskSpace() == False)
   {
	  char szMsg[2048];
	  sprintf(szMsg, "Warning:  not enough disk space in the path %s,  you can either remove unnecessary \n" \
	  "files on the destination drive and type Yes to continue, or click \n" \
	  "CTRL-B to go back and select less products to install\n\n", installer->getServerRoot().data());
	  msg = szMsg;
      me->setText (msg);
   }
   else
   {
      me->hidden();
   }

   return DIALOG_SAME;
}


DialogAction finishSelectionNext (Dialog *me)
{
   const char *buf;
   if (me->isHidden())
   {
      return DIALOG_NEXT;
   }

   buf = me->input();
   return yesNoDefaultNo(buf) ? DIALOG_PREV : DIALOG_NEXT;

}

/////////////////////////////////////////////////////////////////////////
/// Client or Server 
/////////////////////////////////////////////////////////////////////////

DialogInput askClientOrServer(
NULL,
NULL,
INSTALL_SERVERS,

askClientOrServerSetup,
askClientOrServerNext
);

DialogAction askClientOrServerSetup(Dialog *me)
{
   NSString clientorserver_text(getResource(RES_DIALOG_CLIENTORSERVER_TEXT, RES_GLOBAL_BRAND));
   clientorserver_text = clientorserver_text +
           getResource(RES_DIALOG_CLIENTORSERVER_TEXT0, RES_GLOBAL_BRAND) +
           getResource(RES_DIALOG_CLIENTORSERVER_TEXT1, RES_GLOBAL_BRAND) +
           getResource(RES_DIALOG_CLIENTORSERVER_TEXT2, RES_GLOBAL_BRAND) +
           getResource(RES_DIALOG_CLIENTORSERVER_TEXT3, RES_GLOBAL_BRAND);
   me->setText(clientorserver_text);
   me->setPrompt(getResource(RES_DIALOG_CLIENTORSERVER_PROMPT));

   dialogSetup(me, INSTALL_WHICH);
   return DIALOG_SAME;
}

DialogAction askClientOrServerNext(Dialog *me)
{
   const char *buf = me->input();
   Installer *installer = (Installer *) me->manager()->parent();
   InstallInfo *installInfo = me->manager()->getDefaultScript();

   DialogAction rc ;

   if (buf[0] == 0 )
   {
       buf = me->defaultAns();
   }

   if (strcasecmp(INSTALL_SERVERS, buf) && strcasecmp(INSTALL_CLIENTS,buf))
   {
      me->setDefaultAns(INSTALL_SERVERS);
      rc = DIALOG_SAME;
   }
   else
   {
      if (me->isVisited() == False || strcasecmp(buf, installInfo->get(INSTALL_WHICH)))
      {
         if (me->isVisited())
         {
            me->manager()->resetLast();
         }

         if (!strcasecmp(buf, INSTALL_SERVERS))
         {
            askInstallType.registerDialogNext(me->manager());
         }
         else
         {
            askClientRoot.registerDialogNext(me->manager());
         }
      }
      me->visited();
      me->setDefaultAns(buf);
      installInfo->set(INSTALL_WHICH, buf);

      rc = DIALOG_NEXT;
   }

   return rc;

}


////////////////////////////////////////////////////////////////////////
/// AskInstallType
/////////////////////////////////////////////////////////////////////////

DialogInput askInstallType(
NULL,
NULL,

DEFAULT_INSTALL_TYPE,

askInstallTypeSetup,
askInstallTypeNext
);

DialogAction askInstallTypeSetup(Dialog *me)
{
   me->setText(getResource(RES_DIALOG_INSTALLTYPE_TEXT));
   me->setPrompt(getResource(RES_DIALOG_INSTALLTYPE_PROMPT)); 

   dialogSetup(me, INSTALL_TYPE);
   InstallInfo *installInfo = me->manager()->getDefaultScript();
   Installer *installer = (Installer *) me->manager()->parent();
   const char *mode;

   mode = installer->newSuiteSpot()->get("Mode");
   if (mode && strcasecmp(mode, "Allmodes") != 0)
      me->hidden();

   return DIALOG_SAME;
}

DialogAction askInstallTypeNext(Dialog *me)
{
   const char *buf = me->input();
   Installer *installer = (Installer *) me->manager()->parent();
   InstallInfo *installInfo = me->manager()->getDefaultScript();

   DialogAction rc ;

   if (buf[0] == 0 )
   {
       buf = me->defaultAns();
   }

   if (strcasecmp(buf, EXPRESS_TYPE) && strcasecmp(buf, TYPICAL_TYPE) && strcasecmp(buf, CUSTOM_TYPE))
   {
      me->setDefaultAns(TYPICAL_TYPE);
      if (me->isHidden() == True)
         rc = DIALOG_NEXT;
      else
         rc = DIALOG_SAME;
   }
   else
   {
      if (me->isVisited() == False || strcasecmp(installInfo->get(INSTALL_TYPE), buf))
      {
         if (me->isVisited())
         {
            me->manager()->resetLast();
         }
         askServerRoot.registerDialogNext(me->manager());
      }

      me->visited();
      installer->setInstallType((InstallType) atoi(buf));
      me->manager()->setInstallType((InstallType) atoi(buf));
      installInfo->set(INSTALL_TYPE,buf);

      if (me->manager()->installType() == Express)
      {
         installer->newSuiteSpot()->selectDefaults();
      }

      rc = DIALOG_NEXT;
   }

   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// AskServerRoot
/////////////////////////////////////////////////////////////////////////

DialogInput askServerRoot(
NULL,
NULL,

DEFAULT_SROOT,

askServerRootSetup,
askServerRootNext
);

/////////////////////////////////////////////////////////////////////////
/// AskClientRoot
/////////////////////////////////////////////////////////////////////////

DialogInput askClientRoot(
NULL,
NULL,

DEFAULT_CLIENTROOT,

askClientRootSetup,
askServerRootNext
);

DialogAction askServerRootSetup(Dialog *me)
{
   me->setText(getResource(RES_DIALOG_SERVERROOT_TEXT));
   me->setPrompt(getResource(RES_DIALOG_SERVERROOT_PROMPT));

   dialogSetup(me, SERVER_ROOT);

   return DIALOG_SAME;
}

DialogAction askClientRootSetup(Dialog *me)
{
   NSString clientroot_text(getResource(RES_DIALOG_CLIENTROOT_TEXT, RES_GLOBAL_BRAND));
   clientroot_text = clientroot_text +
                     getResource(RES_DIALOG_CLIENTROOT_TEXT0, RES_GLOBAL_BRAND);

   me->setText(clientroot_text);
   me->setPrompt(getResource(RES_DIALOG_CLIENTROOT_PROMPT));

   dialogSetup(me, SERVER_ROOT);

   return DIALOG_SAME;
}

DialogAction askServerRootNext(Dialog *me)
{
   NSString errMsg;
   Installer *installer = (Installer *) me->manager()->parent();
   InstallInfo *installInfo = me->manager()->getDefaultScript();
   DialogAction rc = DIALOG_NEXT;
   int i;

   char *buf = strdup(me->input());

   if (buf[0] == 0)
   {
      free(buf);
      buf = strdup(me->defaultAns());
   }

   while (buf[i=strlen(buf)-1] == '/' && i)
   {
      buf[i] = 0;
   } 

   if (me->isVisited() == False || strcasecmp(installInfo->get(SERVER_ROOT), buf))
   {
      installInfo->set(SERVER_ROOT, buf);
      /*
       *  refresh existing component list
       */
      installer->refreshNewProducts();
      errMsg = installer->checkServerRoot(buf);


      if (errMsg != (char *) NULL)
      {
         errMsg = NSString("ERROR: ") + errMsg;
         DialogAlert alert(errMsg);

         alert.execute();

         rc = DIALOG_SAME;
      }
      else if (!strcasecmp(installInfo->get(INSTALL_WHICH), INSTALL_CLIENTS))
      {
         installer->selectAllClients();
         rc = DIALOG_EXIT;
      }
      else
      {
         Dialog *selectionDialog;

         me->manager()->resetLast();

         installer->checkInstalled();
         selectionDialog = createSelectionDialog(
            installer->newSuiteSpot()->getComponents(),
            "Specify the components you wish to install",
            askSelectionNext,
            getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND));
         selectionDialog->registerDialogLast(me->manager());
      }
   }

   if (rc == DIALOG_NEXT)
   {
      me->visited(); 
   }

   free(buf);

   return rc;
}


DialogAction
verifySelection (Dialog *me, Bool express)
{
   Installer  *installer = (Installer *) me->manager()->parent();
   Components *components;
   NSString errMsg;

   me->getUserData("Components", (long &) components);
   if (getSelectedComponent (me, components, express) == 0)
   {
      return DIALOG_SAME;
   }

   /* Version, compatibility, and security checking */
   errMsg = installer->checkInstallable(components);

   if (errMsg == (char *) NULL)
   {
      errMsg = installer->checkComponentsDependency(installer->getInstallableComponents(), 0);
   }

   if (errMsg != (char *) NULL)
   {
      errMsg = NSString("ERROR: ") + errMsg;
      DialogAlert alert (errMsg);
      alert.execute();
      return DIALOG_SAME;
   }
   return DIALOG_NEXT;
}

DialogAction
askSelectionNext(Dialog *me)
{
   DialogAction rc;
   Bool express = False;

   if (me->manager()->installType() == Express)
   {
      express = True;
   }

   rc = verifySelection(me, express);

   if (rc == DIALOG_NEXT)
   {

      if (express == False)
      {
         /*
          * Take care of module
          */
         Components *products;
         me->getUserData("Components", (long &) products);

         if (products->isTop())
            me->manager()->resetLast();

         for (int i = 0 ; i < products->numComponents(); i++)
         {
            Product *p = products->getComponent(i);

            if (p->isSelected() && p->numComponents() > 0 )
            {
               Dialog *selectionDialog;
               selectionDialog = createSelectionDialog(
                  p->getComponents(),
                  "Specify the components you wish to install",
                  askSelectionNext,
                  getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND),
                  p->get(PKG_NAME));
               selectionDialog->registerDialogLast(me->manager());
            }
         }
         me->visited();

         // Make all necessary components are selected. 
         finishSelection.registerDialogLast(me->manager());

         askHostname.registerDialogLast(me->manager());
      }
      else
      {
         /*
          *  detect machine name in express mode
          */
         me->visited();
         me->manager()->getDefaultScript()->set(MACHINE_NAME, InstUtil::guessHostname());
         askSSUser.registerDialogNext(me->manager());
      }
   }
   return rc;
}


/////////////////////////////////////////////////////////////////////////
/// AskHostName
/////////////////////////////////////////////////////////////////////////
DialogInput askHostname(
NULL,
NULL,

InstUtil::guessHostname(),

askHostNameSetup,
askHostNameNext
);

DialogAction
askHostNameSetup(Dialog *me)
{
   me->setText(getResource(RES_DIALOG_HOSTNAME_TEXT));
   me->setPrompt(getResource(RES_DIALOG_HOSTNAME_PROMPT));

   dialogSetup(me, MACHINE_NAME);
   return DIALOG_SAME;
}

DialogAction
askHostNameNext(Dialog *me)
{
   const char *buf = me->input();
   if (buf[0] == 0)
   {
      buf = me->defaultAns();
   }

   // Bug 605982: Computer name should be 7-bit ascii
   if (!setupIsAsciiBuffer(buf))
   {
	   NSString errMsg = NSString("The computer name must contain 7 bit ascii only.");
	   DialogAlert alert(errMsg);
	   alert.execute();
	   return DIALOG_SAME;
   }

   me->manager()->getDefaultScript()->set(MACHINE_NAME, buf) ;
   if (!strchr(buf, '.'))
   {
       me->showString("WARNING: that is not a fully qualified domain name.");
   }

   askSSUser.registerDialogNext(me->manager());
   askHostname.visited();

   return DIALOG_NEXT;
}

/////////////////////////////////////////////////////////////////////////
/// AskSSUser
/////////////////////////////////////////////////////////////////////////
DialogInput askSSUser(
NULL,
NULL,

DEFAULT_SSUSER,

askSSUserSetup,
askSSUserNext
);

DialogAction
askSSUserSetup(Dialog *me)
{
   Installer *installer = (Installer *) me->manager()->parent();
   const char *s;

   me->setText(getResource(RES_DIALOG_SSUSER_TEXT, RES_GLOBAL_BRAND));
   me->setPrompt(getResource(RES_DIALOG_SSUSER_PROMPT));

   dialogSetup(me, SS_USER_ID);

   // skip license dialog if someone knows how to set it
   s = installer->newSuiteSpot()->get("skipSSUserDialog");
   if (s && !strcasecmp(s, "true"))
   {
      me->hidden();
   }

   return DIALOG_SAME;
}

DialogAction
askSSUserNext(Dialog *me)
{
   NSString ssUser;
   NSString ssGroup;
   DialogAction rc = DIALOG_NEXT;
   NSString errMsg;
   char tmp[XSM_BUF];
   Installer  *installer = (Installer *) me->manager()->parent();
   const char *buf = me->input();

   if (buf[0] != 0)
   {
     ssUser = buf;
   }
   else
   {
     ssUser = me->defaultAns();
   }
   

   if (me->isHidden() == True)
   {
      ssGroup = me->manager()->getDefaultScript()->get(SS_GROUP);
   }
   else
   {
      errMsg = installer->verifySSUser(ssUser);

      if (errMsg != (char *) NULL)
      {
         errMsg = NSString("ERROR: ") + errMsg;
         DialogAlert alert(errMsg);

         alert.execute();
         return DIALOG_SAME;
      }

      // Ask SS Group

      ssGroup = me->manager()->getDefaultScript()->get(SS_GROUP);
      sprintf(tmp, "%s [%s]: ", "System Group", ssGroup.data());
      me->showString(tmp);

      if (me->getInput() == 0)
      {
         return DIALOG_PREV;
      }

      const char *inp = me->input();

      if (inp[0] == 0)
         ssGroup = me->manager()->getDefaultScript()->get(SS_GROUP);
      else
         ssGroup = inp;

      errMsg = installer->verifySSGroup(ssUser, ssGroup);

      if (errMsg != (char *) NULL)
      {
         errMsg = NSString("ERROR: ") + errMsg;
         DialogAlert alert(errMsg);
         alert.execute();
         return DIALOG_SAME;
      }
   }


   Bool askLdap;

   me->manager()->getDefaultScript()->set(SS_USER_ID,ssUser);
   me->manager()->getDefaultScript()->set(SS_GROUP,ssGroup);

   askLdap = installer->needLdapInfo();

   if (askLdap)
   {
      char *localLdapURL = NULL;
      installer->getLocalLdapInfo(&localLdapURL, NULL, NULL);
      if (localLdapURL == NULL)
         askLdapInfo.registerDialogNext(me->manager());
      else
         askLdapUser.registerDialogNext(me->manager());
   }
   else
   {
       me->manager()->getDefaultScript()->remove(CONFIG_LDAP_URL);
       me->manager()->getDefaultScript()->remove(MC_ADMIN_ID);
       me->manager()->getDefaultScript()->remove(MC_ADMIN_PWD);
	    me->manager()->getDefaultScript()->remove(DS_ADMIN_DOMAIN);
       rc = DIALOG_EXIT;
   }

   askSSUser.visited();

   return rc;

}

DialogYesNo removeInstallInfo(
NULL,
NULL,

"Yes",

removeInstallInfoSetup,
removeInstallInfoNext
);

DialogAction removeInstallInfoSetup(Dialog *me)
{
   me->setText(getResource(RES_DIALOG_REMOVEINSTALLINFO_TEXT));
   me->setPrompt(getResource(RES_DIALOG_REMOVEINSTALLINFO_PROMPT));
   return DIALOG_SAME;
}

DialogAction removeInstallInfoNext (Dialog *me)
{
   const char *buf = me->input();
   return yesNoDefaultYes(buf) ? DIALOG_NEXT : DIALOG_TERMINATE;
}
