/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/**********************************************************************
** NAME
**   dialog.h
**
** DESCRIPTION
**   Dialog classes
**
** NOTES
**
** HISTORY
**
** $Log: dialog.h,v $
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:30  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:18  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.5  1999/10/01 20:54:19  cxue
** Simple YesNoDialog for warning
**
** Revision 1.1.1.4  1999/09/28 20:39:06  cxue
** non-virtual destructor warning from linux
**
** Revision 1.1.1.3  1999/09/17 21:21:49  cxue
** Move inline funciton -- answer() to normal function to solve compilation error
**
** Revision 1.1.1.2  1999/09/17 01:48:06  cxue
** answer() to return default value when user hits return
**
** Revision 1.1.1.1  1999/03/04 23:27:12  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.11.6.2  1998/10/15 22:22:10  cxue
** resource framework for custom ui
**
** Revision 1.1.2.11.6.1  1998/10/02 18:05:41  cxue
** Support mode=express|typical|custom|all modes in [general] section. New hidden attribute for Dialog class
**
** Revision 1.1.2.11  1998/04/14 01:11:59  pvo
** 113369 - added pause
**
** Revision 1.1.2.10  1998/03/28 03:29:35  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.9  1997/12/12 21:54:34  pvo
** Various bug fixes.
**
** Revision 1.1.2.8  1997/11/26 01:11:47  pvo
** Added enable/disable back screen capability.
**
** Revision 1.1.2.7  1997/11/16 22:34:15  pvo
** Added back installType.
**
** Revision 1.1.2.6  1997/11/13 19:28:55  pvo
** More new stuffs.
**
** Revision 1.1.2.5  1997/11/08 16:57:17  pvo
** Added Dialog::newPage() for general use.
**
** Revision 1.1.2.4  1997/10/24 03:39:42  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.3  1997/10/22 02:40:54  pvo
** Too many changes to list out here.
**
** Revision 1.1.2.2  1997/09/30 18:40:51  pvo
** More documentation.
**
** Revision 1.1.2.1  1997/09/27 02:47:31  pvo
** Checked in.
**
***********************************************************************/

#ifndef NS_DIALOG_H
#define NS_DIALOG_H

/*
 * Includes
 */
#include <string.h>
#include "nsdefs.h"
#include "ux-util.h"

/*
 ************************************************************************
 * Foward declarations and typedefs.
 ************************************************************************
 */

class Dialog;

//
// Action associated with a Dialog to be carried out by a Dialog Manager
//
typedef enum { DIALOG_SAME,    // Return to the current dialog
               DIALOG_PREV,    // Return to previous dialog
               DIALOG_NEXT,    // Go to next dialog
               DIALOG_EXIT,    // Exit dialog manager
               DIALOG_ERROR,   // Exit - catastrophic error - should not occur
               DIALOG_TERMINATE //Exit - terminate program
             } DialogAction;

//
// Dialog callback
//
typedef DialogAction (*DLG_CALLBACK) (Dialog *);

/*
** CLASS:
**   DialogManager
**
** DESCRIPTION:
**   ABSTRACT class defining a manager who manages the screen dialogs.
**   The Dialog Manager is responsible to sequence through all the
**   dialogs depending on the DialogActions returned by the Dialogs.
**
**   In general, you don't need a dialog manager. It depends on
**   how you implement dialog control. The dialog manager could be
**   just your main program.
**
** NOTE:
**
*/

class DialogManager
{
public:
   DialogManager();

   virtual ~DialogManager();

   void setInstallMode (InstallMode mode)
     { _installMode = mode; }

   InstallMode installMode () const
     { return _installMode; }

   void setInstallType (InstallType type)
     { _installType = type; }

   InstallType installType () const
     { return _installType; }

   /*
    * Call this before starting the the dialogs
    */
   static void enableWinMode();
   static void disableWinMode();
   static void newPage();

   void setDefaultScript(InstallInfo *script);
   InstallInfo *getDefaultScript() const;

   /*
    * Start the dialog manager
    */
   virtual int  start() = 0;

   /*
    * Continue the dialogs where it was left off previously
    */
   virtual int  cont() = 0;

   /*
    * Clear (remove) all dialogs owned by this manager
    */
   virtual void clear() = 0;

   /*
    * Add a dialog to the list managed by this manager
    * right after the current dialog
    */
   virtual void add (Dialog *) = 0;

   /*
    * Add a dialog to the list managed by this manager
    * at the end of the list of dialogs already there
    */
   virtual void addLast (Dialog *) = 0;

   /*
    * Clear all dialogs after the current dialog
    */
   virtual void resetLast () = 0;

   /*
    * Restore previously backed-out dialogs
    */
/*
   virtual void restore() = 0;
*/

   /*
    * Set/Get the parent of this dialog manager
    */
   virtual void setParent(void *) = 0;
   virtual void *parent() const = 0;

private:
   InstallMode _installMode;
   InstallType _installType;

   InstallInfo *_installInfo;

   static Bool _winMode;

};

/*
** CLASS:
**   Dialog
**
** DESCRIPTION:
**   Virtual Screen dialog class.
**   A Dialog is composed of the following:
**     - The header
**     - Text Area 1 - Instruction
**     - Text Area 2 - typically used for Example
**     - Prompt
**     - Default Answer (displayed in enclosed brackets)
**     - Input area
**
**   Associated with each Dialog are three callback functions to be
**   supplied by the Dialog implementer
**     - SetUp: to be executed before the Dialog is displayed
**     - Next : to be executed after the user has entered the input
**     - Back : to be executed when the user wants to go back to
**              the previous dialog.
**   In general, Next() is always needed. Setup() might be needed.
**   Back() is rarely used. Specify NULL if the callbacks are not
**   supplied.
**
** NOTE:
**
*/

class Dialog
{
public:

   /*
   ** Public constructor
   **/
   
   Dialog (const char *text = 0,
           const char *prompt = 0,
           const char *defAns = 0,
           DLG_CALLBACK setup = 0,
           DLG_CALLBACK next = 0,
           DLG_CALLBACK prev = 0);

   virtual ~Dialog () {} ;

  /*
   * MEMBER:
   *   Dialog::initDisplay
   *
   * DESCRIPTION:
   *  This is a static function which must be called first time
   *  to initialize the display and the dialog header
   *
   * INPUTS:
   *   - Product name       : optional. Used to init the header.
   *                          If not supplied, "SuiteSpot" is used.
   * OUTPUTS:
   *   - None
   * RETURN:
   *   - None.
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   None
   * MEMORY:
   *   
   */

   static void initDisplay(const char *productName,const char *title);
   static void initDisplay(const char *productName);

  /*
   * MEMBER:
   *   Dialog::setText
   *   Dialog::setText2
   *   Dialog::setPrompt
   *   Dialog::setDefaultAns
   *
   * DESCRIPTION:
   *  Set the string values of the Text1, Text2, ... areas.
   *
   * INPUTS:
   *   - const char *text
   * OUTPUTS:
   *   - None
   * RETURN:
   *   - None.
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   None
   * MEMORY:
   *   
   */
   void setText (const char *);
   void setText2(const char *);
   void setPrompt (const char *);
   void setDefaultAns (const char *);
   const char *defaultAns() const
     { return _defaultAns; }

  /*
   * MEMBER:
   *   Dialog::enable8BitInput
   *   Dialog::disable8BitInput
   * DESCRIPTION:
   *   Enable/disable conversion of input from local to UTF8
   *   
   *
   * INPUTS:
   *   int length
   * OUTPUTS:
   *   None
   * RETURN:
   *   None
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   Lenght must be positive and smaller or equal to 1024.
   * MEMORY:
   *
   */
   void enable8BitInput()
      {_enable8BitInput = True; }
   void disable8BitInput()
      {_enable8BitInput = False; }
  /*
   * MEMBER:
   *   Dialog::setInputLen
   * DESCRIPTION:
   *   Set the maximum length of the input buffer. By default,
   *   the input buffer is set to 1024 bytes.
   *
   * INPUTS:
   *   int length
   * OUTPUTS:
   *   None
   * RETURN:
   *   None
   * EXCEPTION THROWN
   *   None
   * RESTRICTIONS:
   *   Lenght must be positive and smaller or equal to 1024.
   * MEMORY:
   *  
   */
   void setInputLen(int length);

  /*
   * MEMBER:
   *   Dialog::setSetupAction
   *   Dialog::setNextAction
   *   Dialog::setPrevAction
   *
   * DESCRIPTION:
   *   Set the callbacks.
   *
   * INPUTS:
   *   Callback function DLG_CALLBACK
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   void setSetupAction(DLG_CALLBACK f);
   void setNextAction(DLG_CALLBACK f);
   void setPrevAction(DLG_CALLBACK f);

  /*
   * MEMBER:
   *   Dialog::setUserData
   *   Dialog::getUserData
   *
   * DESCRIPTION:
   *   Set the callbacks.
   *
   * INPUTS:
   *   Callback function DLG_CALLBACK
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   void setUserData(const char *name, const char *value);
   void setUserData(const char *name, long value);
   const char *getUserData(const char *name);
   int getUserData(const char *name, long & value);

  /*
   * MEMBER:
   *   Dialog::registerDialog
   *   Dialog::manager()
   * DESCRIPTION:
   *   Register myself with the given Dialog Manager.
   *   Get my Dialog Manager.
   *
   * INPUTS:
   *   DialogManager *
   *   flag: arbitrary value used to communicate with DialogManager
   *     on how to manage this dialog. Note that user has to
   *     implement the DialogManager thus have to interprete this
   *     value accordingly. A value of 0 should be the default.
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */
   void registerDialogNext(DialogManager *);
   void registerDialogLast(DialogManager *);

   DialogManager *manager()
     { return _dlgMgr; }

   /////////////////////////////////////////////////////////////////
   // Display functions
   ///////////////////////////////////////////////////////////////// 

  /*
   * MEMBER:
   *   Dialog::show
   *   Dialog::showText
   *   Dialog::showString
   *   Dialog::showProgress
   * DESCRIPTION:
   *   Display the Dialog.
   *
   *   Dialog::show                     Shows the dialog
   *
   *   Dialog::showText                 Shows the text areas only and
   *                                    wait for any input. 
   *
   *   Dialog::showString               Displays a given string and returns.
   *
   *   Dialog::showProgress             Displays a given string at the same
   *                                    location.
   *
   * INPUTS:
   *   DialogManager *
   * OUTPUTS:
   *
   * RETURN:
   *   Dialog::show returns DialogAction NEXT_DIALOG, PREV_DIALOG, 
                  and ERROR_DIALOG
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *   DialogManager::enableWinMode() must have been called.
   *
   * MEMORY:
   * 
   */

   DialogAction show();
   void showText();
   static void showString(const char *);
   static void showProgress(const char *);
   static void pause(const char * = NULL);

   /*
    * Disable CTRL-B. Short-lived, only valid per one
    * input prompt
    */
   static void disableBackPage();

  /*
   * MEMBER:
   *   Dialog::execute
   * DESCRIPTION:
   *   Execute a Dialog. Will call Dialog::show.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *
   * RETURN:
   *   NEXT_DIALOG, PREV_DIALOG, SAME_DIALOG, EXIT_DIALOG, ERROR_DIALOG
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *   DialogManager::enableWinMode() must have been called.
   *
   * MEMORY:
   *
   */
   virtual DialogAction execute() = 0;

  /*
   * MEMBER:
   *   Dialog::input
   * DESCRIPTION:
   *   Returns the input buffer
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   const char *input() const
      { return _buf; }

  /*
   * MEMBER:
   *   Dialog::answer
   * DESCRIPTION:
   *   Returns the input buffer or defaultAns if no input available
   *   
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   const char *answer() const;

  /*
   * MEMBER:
   *   Dialog::isVisited
   * DESCRIPTION:
   *   Returns True if the Dialog has been visited, otherwise False
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *   True/False
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   Bool isVisited() const
      { return _visited; }

  /*
   * MEMBER:
   *   Dialog::visited
   *   Dialog::unVisited()
   * DESCRIPTION:
   *   Indicate that the Dialog has been visited
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   void visited()
      { _visited = True; }
   void unVisited()
      { _visited = False; }

  /*
   * MEMBER:
   *   Dialog::isHidden
   * DESCRIPTION:
   *   Returns True if the Dialog is hidden. otherwise False
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *   True/False
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   Bool isHidden() const
      { return _hidden; }

  /*
   * MEMBER:
   *   Dialog::hidden()
   *   Dialog::unHidden()
   * DESCRIPTION:
   *   Indicate if Dialog needs to be displayed
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   void hidden()
      { _hidden = True; }
   void unHidden()
      { _hidden = False; }


  /*
   * MEMBER:
   *   Dialog::getInput
   * DESCRIPTION:
   *   Get the input. The input buffer is filled with the input as
   *   appropriate.
   * INPUTS:
   *
   * OUTPUTS:
   *
   * RETURN:
   *
   * EXCEPTION THROWN
   *
   * RESTRICTIONS:
   *
   * MEMORY:
   *
   */
   int getInput();

  /*
   * MEMBER:
   *   Dialog::getPassword
   * DESCRIPTION:
   *   A specialized function similar to getEvent but the input
   *   is not echoed to the screen.
   *
   * INPUTS:
   *   
   * OUTPUTS:
   *   
   * RETURN:
   *  
   * EXCEPTION THROWN
   * 
   * RESTRICTIONS:
   *
   * MEMORY:
   *  
   */

   int getPassword();

protected:

   /*
    * Callback for RETURN (next)
    *   Return values:
    *     True: OK
    *     False: Error
    */
   DialogAction next();

   /*
    * Callback for BACK (prev)
    *   - Should always go back
    */
   void prev();

   Bool _visited;

   Bool _hidden;

   Bool _show;

   Bool _enable8BitInput;

   char     _buf[MED_BUF];

private:

   static NSString _title;
   static NSString _product;

   NSString _text;
   NSString _text2;
   NSString _prompt;
   NSString _defaultAns;

   int      _bufLen;

   DLG_CALLBACK _setupAction;
   DLG_CALLBACK _nextAction;
   DLG_CALLBACK _prevAction;

   DialogManager   *_dlgMgr;

   NVPair   _userData;
};

//////////////////////////////////////////////////////////////////////////
// Some canned class for easy use
//////////////////////////////////////////////////////////////////////////

/*
** CLASS:
**   DialogYesNo
**
** DESCRIPTION:
**   A Dialog that just collects a Yes/No answer.
**
** NOTE:
**
*/

class DialogYesNo:public Dialog
{
public:
   DialogYesNo ()
     {}
   DialogYesNo(const char *text,
               const char *prompt,
               const char *defaultAns,
           DLG_CALLBACK setup = 0,
           DLG_CALLBACK next = 0,
           DLG_CALLBACK prev = 0);
   /*
    *  Simplified version of prvious DialogYesNo
    *  return DIALOG_NEXT on YES, DIALOG_SAME on NO  
    */
   DialogYesNo(const char *text,
               const char *defaultAns);

   DialogAction execute();
private:
   DialogAction processInput();
   Bool _default;
};

/*
** CLASS:
**   DialogInput
**
** DESCRIPTION:
**   A typical Dialog
**
** NOTE:
**
*/

class DialogInput:public Dialog
{
public:
   DialogInput ()
      {}
   DialogInput (const char *text,
                const char *prompt,
                const char *defaultAns,
           DLG_CALLBACK setup = 0,
           DLG_CALLBACK next = 0,
           DLG_CALLBACK prev = 0,
           Bool allow8BitInput = False);

   DialogAction execute();
};

/*
** CLASS:
**   DialogAlert
**
** DESCRIPTION:
**   A "popup" dialog. Popup a message and awaits any input then returns
**   to the current dialog.
**
** NOTE:
**
*/

class DialogAlert:public Dialog
{
public:
   DialogAlert()
      {}
   DialogAlert(const char *message);
   DialogAction execute();
};

/*
** CLASS:
**   DialogPwdEntry
**
** DESCRIPTION:
**   Dialog to collect password
**
** NOTE:
**
*/

//class DialogPwdEntry:public Dialog
//{
//public:
 //  DialogPwdEntry ();
  // DialogPwdEntry (const char *prompt);
  // DialogAction execute();
//};

/*
** CLASS:
**   DialogProgress
**
** DESCRIPTION:
**   Display the progress of certain activities.
**
** NOTE:
**
*/

class DialogProgress:public Dialog
{
public:
   DialogProgress():_newPage(False)
      {}
   ~DialogProgress()
      {}
   DialogAction execute();

   /*
    * Display the progress string
    */
   void showProgress(const char *text);

private:
   Bool _newPage;
};

#endif
