/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME:
**   uninstaller.cc
**
** DESCRIPTION:
**   Uninstallation Program for servers
**
** NOTES:
**
** HISTORY:
** $Log: uninstaller.cc,v $
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.5  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.4  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.3  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/03 18:24:10  nhosoi
** rebranded setupsdk as brandx; updated the version number to 7.0
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.13.4.2  2002/04/15 18:02:20  djani
** Bug 601242: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.13.4.1  2002/04/13 00:08:57  djani
** Bug 540776: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.13  2001/11/16 00:18:58  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.12  2001/09/20 21:36:38  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.11  2000/07/12 23:24:12  peterz
** support -force during silent uninstallation to remove everyting even when ldap info are not provided
**
** Revision 1.1.1.10  2000/06/02 22:11:32  peterz
** on second thought, change iPlanet Server to just iPlanet
**
** Revision 1.1.1.9  2000/06/02 22:04:53  peterz
** change SuiteSpot to iPlanet Server
**
** Revision 1.1.1.8  2000/02/09 00:01:34  cxue
** silent uninstall should take -l option
**
** Revision 1.1.1.7  1999/11/09 19:30:39  cxue
** uninst -s without ldapurl
**
** Revision 1.1.1.6  1999/10/13 00:19:27  cxue
** chnage <serverroot> and go forward
**
** Revision 1.1.1.5  1999/07/29 00:23:57  svbld
** Merged the admin42_branch back onto the admin_branch..
**
** Revision 1.1.1.3.2.2  1999/06/25 00:46:51  shihcm
** remove string: Version 4.0
**
** Revision 1.1.1.3.2.1  1999/06/16 22:48:26  cxue
** Calling fgets instead of gets to get rid of warnings on Linux
**
** Revision 1.1.1.3  1999/06/07 17:49:32  cxue
** remove uninstall cache file when it's done
**
** Revision 1.1.1.2  1999/04/16 17:12:08  cxue
** bug 347119: uninstaller doesn't call pre- post- uninstall program
**
** Revision 1.1.1.1  1999/03/04 23:27:08  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.28.4.9  1999/02/04 02:24:47  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.28.4.8  1998/12/04 02:49:23  cxue
** billx uninstallation and customizable destination support
**
** Revision 1.1.2.28.4.7  1998/11/18 02:37:54  cxue
** user specified ldapURL for SSL enable config directory during silent uninstallation
**
** Revision 1.1.2.28.4.6  1998/11/10 02:41:31  cxue
** id/password authetication before starting silent uninstall
**
** Revision 1.1.2.28.4.5  1998/11/02 21:37:28  cxue
** product name from resource
**
** Revision 1.1.2.28.4.4  1998/10/29 03:13:36  cxue
** move dialog resource from dl-common.cc to uxres_data.cc
**
** Revision 1.1.2.28.4.3  1998/10/28 17:55:51  cxue
** support userdefined default setting
**
** Revision 1.1.2.28.4.2  1998/10/15 22:22:07  cxue
** resource framework for custom ui
**
** Revision 1.1.2.28.4.1  1998/09/09 18:17:00  cxue
** Silent uninstall support
**
** Revision 1.1.2.28  1998/07/02 03:30:33  pvo
** Some minor name changes
**
** Revision 1.1.2.27  1998/07/01 01:25:48  pvo
** More I18n bugs. toLocal before write install.info
**
** Revision 1.1.2.26  1998/06/01 21:20:09  pvo
** 122446 - Uninstall prematurely deletes itself
**
** Revision 1.1.2.25  1998/05/01 23:24:43  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.24  1998/04/14 01:12:13  pvo
** 113369 - added pause
**
** Revision 1.1.2.23  1998/04/07 18:48:06  pvo
** 112703 - Uninstall based on dependency order
**
** Revision 1.1.2.22  1998/04/01 02:42:32  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.21  1998/03/24 22:29:45  pvo
** 110961 - Setup cannot handle long directory name.
**
** Revision 1.1.2.20  1998/03/13 21:24:48  pvo
** 109929 - Allow uninstall even if DS is down
**
** Revision 1.1.2.19  1998/03/13 05:00:38  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.18  1998/03/10 02:49:17  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.17  1998/03/09 22:09:38  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.16  2000/03/03 21:39:59  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.15  1998/02/26 00:14:01  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.14  1998/02/15 22:06:42  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.13  1998/02/04 00:01:54  pvo
** 104824 - DEC uninstall goes out of control
**
** Revision 1.1.2.12  1998/01/29 21:52:24  pvo
** Bug 105041 - add -h (help) screen for setup
** Bug 104968 - change default LDAP suffix to domain.com rather than example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.11  1998/01/27 23:54:49  pvo
** Fixed OSF1 compile.
**
** Revision 1.1.2.10  1998/01/09 03:27:32  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.9  1997/12/18 02:10:23  pvo
** Fixed problem on HPUX with tmpnam.
**
** Revision 1.1.2.8  1997/12/13 02:40:10  pvo
** Support for new LDAP schema.
**
** Revision 1.1.2.7  1997/12/12 21:54:22  pvo
** Various bug fixes.
**
** Revision 1.1.2.6  1997/11/26 02:03:02  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.5  1997/11/13 19:28:45  pvo
** More new stuffs. Changed .pi to .inf.
**
** Revision 1.1.2.4  1997/11/11 23:17:39  pvo
** Implemented preUninstall and postUninstall.
**
** Revision 1.1.2.3  1997/11/11 05:41:11  pvo
** Added dependency checking for uninstallation.
**
** Revision 1.1.2.2  1997/11/09 19:30:13  pvo
** More code for uninstallation. Changed ns-remove.cc to uninstall.cc.
** Removed second password prompt when asking Ldap user.
**
** Revision 1.1.2.1  1997/11/08 16:47:49  pvo
** New. Uninstallation support.
**
**
*********************************************************************/

extern "C" {
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>
#include <dirent.h>

}

#include "ldapu.h"
#include "ux-util.h"
#include "dl-common.h"
#include "wizard.h"
#include "uninstaller.h"
#include "dl-uninst.h"
#include "uxres.h"

extern int deleteEmptyDirRecursively(const char*);

const char *SelectionText=
"Following is the list of %s\n"
"components currently installed on your machine.";

UnInstaller::UnInstaller (int argc, char **argv)
: _unInstallMode(Interactive)
{
   char buf[BIG_BUF];
   NSString dir = InstUtil::dirName(argv[0]);
   _baseDir = InstUtil::getCurrentDir();

   if (!strcmp(dir.data(), "."))
   {
      _serverRoot = _baseDir;
   }
   else if (dir[0] != '/')
   {
      chdir(dir.data());
      _serverRoot = InstUtil::getCurrentDir();
      chdir(_baseDir.data());
   }
   else
   {
      _serverRoot = dir;
   }

   _installInfo = new InstallInfo();

   _infoFile = tmpnam(buf);

   initDefaultAnswer(_installInfo);
   
   getOptions(argc, argv);
}

void
UnInstaller::getOptions(int argc, char **argv)
{
   int opt;
   int i = 0;
   char *t;
   int force = 0;
   const char *fileName = NULL;
   char *uid = NULL, *pwd = NULL;
   char *ldapURL = NULL, *ldapUser = NULL, *installDN = NULL;

   _silent = False;

   while ((opt = getopt(argc,argv, "?hsf:u:p:l:")) != -1)
   {
      switch (opt)
      {
         case 's':
            _silent = True;
           _unInstallMode = Silent;
            break;
         case 'f':
            fileName = optarg;
            break;
         case 'u':
            uid = strdup(optarg);
            break;
         case 'p':
            pwd = strdup(optarg);
            break;
         case 'l':
            ldapURL = strdup(optarg);
            break;
         case 'h':
            printf("%s Uninstall Program\n", (const char *)getResource(RES_GLOBAL_PRODUCT));
	         printf("Usage: uninstall [-s -u <uid> -p <password -l <LdapURL>] [-h] [-force]\n");
            printf("Options:\n");
            printf("   -h            print out this message\n");
            printf("   -s            silent uninstallation mode\n");
            printf("   -u <uid>      server configuration admin ID\n");
            printf("   -p <password> server configuration admin password\n");
            printf("   -l <LdapURL>  configuration Ldap URL\n");
            printf("   -force	force silent uninstall even with wrong admin ID and admin password\n");
            exit(0);
         default:
           break;
       }
   }

   if (_silent == False)
      return;

   for(i = 0; i<argc; i++)
   {
   	if(strcmp(argv[i], "-force") == 0)
	{
		force = 1;
		break;
	}
   }

   //if (ldapURL || uid || pwd)              
   //{
     // check if it requires configuration id and password
     if (ldapURL == NULL)
        getDefaultLdapInfo (_serverRoot, &ldapURL, &ldapUser, &installDN);
     else
        getDefaultLdapInfo (_serverRoot, NULL, &ldapUser, &installDN);

     if (ldapURL == NULL)
        return ;

     if ((uid == NULL || pwd == NULL) && !force)
     {
        printf("ERROR: please specify configuration admin ID and password\n");
        exit(0);
     }
   
     if ((strcasecmp(ldapURL, "ldaps") == 0) && !force)
     {
        printf(
           "ERROR: can't run silent installation due to default SSL enabled\n"
           "Config Directory. Please specify Non-SSL enabled Config Directory\n"
           "through command line option -l <LdapURL>\n");
        exit(0);
     }

     // check for valid id/password
     LdapError ldapError;
     Ldap *ldap;

     ldap = new Ldap(ldapError, ldapURL, uid, pwd, (char *)NULL, (char *)NULL);

     if ((ldapError != OKAY) && !force)
     {
        printf("ERROR: authetication failure. Please re-enter configuration ID and password\n");
        exit(0);
     }     

     delete ldap;

     _installInfo->set(MC_ADMIN_ID, uid);
     _installInfo->set(MC_ADMIN_PWD, pwd);

     _installInfo->set(CONFIG_LDAP_URL, ldapURL);
     _installInfo->set(DS_ADMIN_DOMAIN, installDN);
   //}

   /*
    *  In case -f is specified, need to set filename to _installInfo.
    *  and change init() to read it from there.
    */
}

int
UnInstaller::init()
{
   char buf[BIG_BUF];
   NSString sourceDir;
   NSString infName = SUITESPOT_INF;
   int rc = 0;

   if (_serverRoot == (char *) NULL)
   {
      sprintf(buf, "ERROR: please specify a server root\n");
      rc = -1;
   }
   else
   {
      sourceDir = _serverRoot + "/" + SETUP_DIR;

      if (InstUtil::isServerRoot(_serverRoot) == False)
      {
         sprintf(buf, "ERROR: %s is not a server root\n", _serverRoot.data());
         rc = -1;
      }
      else if (_newSuiteSpot.retrieveSuiteSpot(sourceDir, infName) != 0)
      {
         sprintf(buf,"ERROR: uninstallation database not found\n");
         rc = -1;
      }
      else if (_newSuiteSpot.numComponents() == 0)
      {
         sprintf(buf, "ERROR: nothing to remove\n");
         rc = -1;
      }
      else
      {
         _newSuiteSpot.getComponents()->sortComponents();
         _newSuiteSpot.select();
         _newSuiteSpot.getAllComponents(*_newComponents); // Flatten the structure
         _newComponents->sortComponents();
      }

   }

   if (rc)
   {
      printf(buf);
      return rc;
   }

   // init resource, so we can show custom dialogs
   initResource(_newSuiteSpot.package());

   initDialog();

   return 0;
}

void
UnInstaller::initDefaultAnswer(InstallInfo *info)
{
   info->set(SERVER_ROOT, _serverRoot);
   info->set(MACHINE_NAME, InstUtil::guessHostname());

   if (info->get(CONFIG_LDAP_URL) == NULL)
   {
      info->set(CONFIG_LDAP_URL, NSString("ldap://") 
               + info->get(MACHINE_NAME)
               + ":389/"
               + "o=" + InstUtil::guessDomain());

   }

   info->set(MC_ADMIN_ID, DEFAULT_ADMINUSER);
   info->set(MC_ADMIN_PWD, DEFAULT_ADMINPWD);
}

int
UnInstaller::initDialog()
{
   Dialog::initDisplay((_newSuiteSpot.package())->get(SETUP_INF_NAME),
                       (_newSuiteSpot.package())->get(SETUP_INF_VENDOR));

   // Create the dialog manager
   _dialogManager = new Wizard();
   _dialogManager->setParent(this);

   _dialogManager->setDefaultScript(_installInfo);

   _dialogManager->enableWinMode();

   return 0;
}

void
UnInstaller::run()
{
   Dialog *selectionDialog;
   int rc = 0;
   Bool clientOnly = True;
   int i;
   Product *p;

   if (_silent == True)
   {
      for (i = 0; i < _newSuiteSpot.numComponents(); i++)
      {
         /*
          * On silent mode, select all components for removal
          */
         p = _newSuiteSpot.getComponent(i);
         p->selectAll();
      }
   }
   else
   {
      /*
       * First check whether this is a client-only installation
       */
      Components *q = getInstallableComponents();

      for (i = 0; i < q->numComponents(); i++)
      {
         p = q->getComponent(i);
         const char *s = p->get(PKG_ISMCC);
         if (s == NULL || strcasecmp(s, "True"))
         {
            clientOnly = False;
            q = _newSuiteSpot.getComponents();
            break;
         }
      }
      
      /*
       * Setup first dialog
       */
      if (clientOnly)
      {
         for (i = 0; i < _newSuiteSpot.numComponents(); i++)
         {
            _newSuiteSpot.getComponent(i)->select();
         }
      }

      selectionDialog = createSelectionDialog(
         q,
         "Select the components you wish to uninstall (default: all)",
         askSelectionNext,
         getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND),
         "Fedora");

      selectionDialog->registerDialogLast(_dialogManager);

      rc = _dialogManager->start();
   }

   if (rc == 0)
   {
	  // set LD_LIBRARY_PATH env var for solving messaging server bugs
	  char *oldpath = getenv("LD_LIBRARY_PATH");
	  static char newpath[BIG_BUF];
	  strcpy(newpath, "LD_LIBRARY_PATH=");
	  if (oldpath)
	  { 
		strcat(newpath, oldpath);
		strcat(newpath, ":");
	  }
	  strcat(newpath, _serverRoot.data());
	  strcat(newpath, "/lib");
	  if (putenv(newpath))
	  {
		printf("ERROR:Could not set LD_LIBRARY_PATH\n");
	  }


      chdir("/tmp");

      _dialogManager->newPage();

      unInstall();

      chdir(_baseDir.data());

      _dialogManager->disableWinMode();

	  // Reset the LD_LIBRARY_PATH back to its original value
	  strcpy(newpath, "LD_LIBRARY_PATH=");
	  if (oldpath)
	  {
		strcat(newpath, oldpath);
	  }
	  if (putenv(newpath))
	  {
		printf("ERROR:Could not reset LD_LIBRARY_PATH\n");
	  }

      printf("\nUninstallation completed. UnInstall might have been unable to\n"
"remove some files. Please check and remove them manually.\n");
   }

   /*
    * remove installation cache
    */
   if (_infoFile != (char *) NULL)
      unlink(_infoFile.data());

   return;
}

int
UnInstaller::unInstall()
{
   Product *package, *product;
   int err = 0;
   int i, j;

   _installInfo->setFormat(1);
   _installInfo->toLocal();
   _installInfo->write(_infoFile);

   /*
    * For uninstallation, has to traverse the components bottom up
    */
   for (i = _newSuiteSpot.numComponents(); i > 0; i--)
   {
       package = _newSuiteSpot.getComponent(i-1);

       if (package->isSelected())
       {
           for (j = package->numComponents(); j > 0; j--)
           {
              product = package->getComponent(j-1);

              if (product->isSelected())
              {
                 unInstallComponent(product);
              }
          }
          // Done with this package, see if there are any 
          // hidden components that need to be removed
          unInstallHiddenComponents(*package);

          DialogManager::enableWinMode();
      }

   }

   // Done with all packages, see if any hidden packages that
   // need to be removed
   unInstallHiddenComponents(_newSuiteSpot);

   updateSuiteSpotInfo();

   return err;

}

void
UnInstaller::unInstallHiddenComponents(Product & package)
{
   int j;
   Product *product;
   Product *childProduct;

   if (package.getComponents()->hasVisible() == False)
   {
      for (j=0; j < package.numComponents(); j++)
      {
         product = package.getComponent(j);
         if (product->numComponents())
         {
            for (int k = 0; k < product->numComponents(); k++)
            {
               childProduct = product->getComponent(k);
               if (childProduct->isVisible() == False && childProduct->isSelected() == False)
               {
                  childProduct->select();
                  unInstallComponent(childProduct);
               }
            }
         
         }         
         else
         {
            if (product->isVisible() == False)
            {
               product->select();
               unInstallComponent(product);
            }
         }       
      }
   }
}
   
void
UnInstaller::unInstallComponent(Product *product)
{
   DialogProgress dl;
   int err = 0;
   const char *prog;
   char tmp[XSM_BUF];

   DialogManager::disableWinMode();
   err = product->preUnInstall(_serverRoot, _infoFile);
   
   if (err)
   {
      printf("ERROR. Failure uninstalling %s. Do you want to continue [n]? ", product->get(PKG_NAME));
      fflush(stdout);
      fgets(tmp, sizeof(tmp), stdin);
      if (tmp[0] == 'n' || tmp[0] == 'N' || tmp[0] == '\0')
      {
         product->skip(True);
      }
      else
      {
         err = 0;
      }
   }

   if (!err)
   {
      DialogManager::enableWinMode();

      Dialog::showString(NSString ("Uninstalling ") + product->get(PKG_NAME) + "...\n");

      err = product->unInstall(_serverRoot);

      if (err)
      {
         DialogManager::disableWinMode();

         printf("ERROR. Failure uninstalling %s. Do you want to continue [n]? ", product->get(PKG_NAME));
         fflush(stdout);
         fgets(tmp, sizeof(tmp), stdin);
         if (tmp[0] == 'n' || tmp[0] == 'N' || tmp[0] == '\0')
         {
            product->skip(True);
         }
         else
         {
            err = 0;
         }
      }

      if (!err)
      {
         DialogManager::disableWinMode();
         err = product->postUnInstall(_serverRoot, _infoFile);
         DialogManager::enableWinMode();
      }

      prog = product->package()->get(PKG_POSTUNINSTALL);

      if (prog != NULL && *prog)
      {
         unlink(prog);
	   }
   }
}

void
UnInstaller::updateSuiteSpotInfo()
{
   Components *products;
   Product *product;
   PackageInfo *suitespot;
   PackageInfo *component;
   int i, moduleCount;
   NSString setupDir = _serverRoot + "/" + SETUP_DIR;
   NSString infName;
   const char *componentName;

   suitespot = _newSuiteSpot.package();
   products =  _newSuiteSpot.getComponents();

   for (i = 0; i < products->numComponents(); i++)
   {
      product = products->getComponent(i);
      if (product->isSelected() && product->skipped() == False)
      {
         // At this level, product may be selected by there might be
         // some modules not deleted

         component = product->package();

         component->delList(PKG_COMPONENTS);
         componentName = component->get(PKG_NICKNAME);

         if (product->numComponents());
         {
            // Go down to sub-component level
            Components *modules = product->getComponents();
            moduleCount = 0;
            for (int j = 0; j < product->numComponents(); j++)
            {
               Product *module = modules->getComponent(j);
               if (module->isSelected() == True && module->skipped() == False)
               {
                  moduleCount++;
               }
               else
               {
                  // Not deleted, add it back to the component list
                  component->add(PKG_COMPONENTS, module->package()->get(PKG_NICKNAME));
               }
            }
         }

         if (moduleCount == product->numComponents())
         {
            // Remove residues
            infName = setupDir + "/" + component->get("InfFile");
            unlink(infName.data());
            infName = setupDir + "/" + componentName;
            rmdir(infName.data());

            suitespot->delListItem(PKG_COMPONENTS, componentName);

         }
         else
         {
            component->setFormat(1);
            component->write();
         }
      }
   }

   if (suitespot->numListItems(PKG_COMPONENTS) == 0)
   {
      infName = setupDir + "/" + SUITESPOT_INF;
      unlink(infName.data());

      infName = setupDir + "/" + SETUP_INSTALL_LOG;
      unlink(infName.data());
   }
   else
   {
      suitespot->setFormat(1);
      suitespot->write();
   }

   rmdir(setupDir.data());

   // Clean up server root
   if (serverRootEmpty())
   {
      infName = _serverRoot + "/" + ".suitespot";
      unlink(infName.data());

      infName = _serverRoot + "/" + UNINSTALL_INF;
      unlink(infName.data());

      rmdir (_serverRoot.data());
   }
   else
   {
	  //Need to do this as some empty directories will still be hanging around
	  //Example : httpacl of Directory Server
	  deleteEmptyDirRecursively(_serverRoot);
   }
}

Bool
UnInstaller::serverRootEmpty() const
{
   DIR *dirp;
   struct dirent *dir;
   Bool ret = True;

   dirp = opendir(_serverRoot.data());

   while ((dir = readdir(dirp)) != NULL)
   {
      if (strcmp(dir->d_name, ".") != 0 &&
          strcmp(dir->d_name, "..") != 0 &&
          strcmp(dir->d_name, ".suitespot") != 0 &&
          strcmp(dir->d_name, UNINSTALL_INF) != 0)
      {
         ret = False;
         break;
      }
   }
   return ret;
}

