/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/**********************************************************************
** NAME
**   ux-setup.h
**
** DESCRIPTION
**
** NOTES:
**
** HISTORY
**
** $Log: ux-setup.h,v $
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.2.4.1  2002/04/13 00:45:39  djani
** Bug 389570: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.2  2001/11/16 00:18:58  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.1  1999/03/04 23:27:09  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.10.8.4  1999/02/04 02:24:48  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.10.8.3  1998/11/06 20:24:55  cxue
** getUserDefault return reference to NVPair
**
** Revision 1.1.2.10.8.2  1998/10/28 17:55:52  cxue
** support userdefined default setting
**
** Revision 1.1.2.10.8.1  1998/10/15 22:22:08  cxue
** resource framework for custom ui
**
** Revision 1.1.2.10  1998/03/09 22:09:39  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.9  2000/03/03 21:40:00  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.8  1998/02/26 04:02:00  pvo
** Fixed break in download.cc due to the call to set_ldap_option() in libinstall.
** Also try to detect input error when asking for installDN and try to
** create installDN if possible in case the user has the privilege to create
** it.
**
** Revision 1.1.2.7  1998/02/24 00:23:08  pvo
** Support for topology changes.
**
** Revision 1.1.2.6  1998/02/04 00:01:55  pvo
** 104824 - DEC uninstall goes out of control
**
** Revision 1.1.2.5  1998/01/29 21:52:24  pvo
** Bug 105041 - add -h (help) screen for setup
** Bug 104968 - change default LDAP suffix to domain.com rather than example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.4  1998/01/22 04:51:30  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.3  1997/12/12 21:54:24  pvo
** Various bug fixes.
**
** Revision 1.1.2.2  1997/12/02 04:28:29  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.1  1997/11/26 02:03:04  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.3  1997/11/13 19:28:43  pvo
** More new stuffs. Changed .pi to .inf.
**
** Revision 1.1.2.2  1997/11/11 23:17:38  pvo
** Implemented preUninstall and postUninstall.
**
** Revision 1.1.2.1  1997/11/08 16:47:52  pvo
** New. Uninstallation support.
**
***********************************************************************/

#ifndef NS_UXSETUP_H
#define NS_UXSETUP_H

#include "nsdefs.h"
#include "product.h"
#include "uxres.h"

/*
 * Download-related stuffs
 */
extern const char *SITE_INFO_FILE;
extern const char *DOWNLOAD_INFO_FILE;
extern const char *PLATFORM_INFO_FILE;
/*
 * Download information file constants
 */
extern const char *NETSCAPE_URL;

extern const char *SYS_VERSION;
extern const char *DOWNLOAD_ROOT;
   extern const char *DEFAULT_DNLD_ROOT;

extern const char *DOWNLOAD_OPTION;
   extern const char *DOWNLOAD_ONLY;
   extern const char *DOWNLOAD_AND_INSTALL;

extern const char *DOWNLOAD_SOURCE;
   extern const char *FROM_NETSCAPE;
   extern const char *FROM_INTRANET;

extern const char *SOURCE_URL;
extern const char *NETSCAPE_SOURCE_URL;
extern const char *INTRANET_SOURCE_URL;

extern const char *PLATFORM;

class Wizard;

/*
** CLASS:
**   Setup
**
** DESCRIPTION:
**   The SuiteSpot Installer
**
**
*/
class SetUp
{
public:

  SetUp();

  Product *newSuiteSpot();
  Product *oldSuiteSpot();

  int getInstalledProducts(const NSString &serverRoot);

  int existingLdapCheck();

  const NSString &bindDN() const
      { return _bindDN; }
  const NSString &bindPwd() const
      { return _bindPwd; }

  NSString verifyLdapServer(const char *ldapURL);
  NSString verifyLdapUser(const char *ldapURL, const char *ldapUser, const char *ldapPwd);

  Components *getInstallableComponents()
      { return _newComponents; }

  Components *getInstalledComponents()
    { return _oldComponents; }


  NVPair getInstalledConfigInfo()
      { return _existingInfo; };

  NVPair *getUserDefault()
      { return &_userDefault; };

  void terminate(int errcode);
  NSString getServerRoot()
	  { return _serverRoot; }

private:
  void initUserDefault();

protected:

  NSString _workDir;           // Working directory - where I am now
  NSString _tmpDir;            // TMP directory - where things are download
  NSString _baseDir;           // Directory where installer located

  NVPair _userDefault;

  NSString _serverRoot;
  NVPair _existingInfo;

  NVPair *_installInfo;
  NSString _infoFile;

  Wizard *_dialogManager;

  Product _newSuiteSpot;
  Components *_newComponents;

  Product _oldSuiteSpot;
  Components *_oldComponents;

  NSString _bindDN;
  NSString _bindPwd;

};

#endif
