/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/**********************************************************************
** NAME
**   wizard.cc
**
** DESCRIPTION
**
** NOTES
**
** HISTORY
**
** $Log: wizard.cc,v $
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.3  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.2  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.1  1999/03/04 23:27:10  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.9.12.1  1998/10/02 18:05:38  cxue
** Support mode=express|typical|custom|all modes in [general] section. New hidden attribute for Dialog class
**
** Revision 1.1.2.9  1997/12/12 21:54:24  pvo
** Various bug fixes.
**
** Revision 1.1.2.8  1997/12/02 04:28:29  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.7  1997/11/26 02:03:04  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.6  1997/11/08 16:52:04  pvo
** UnInstallation support.
**
** Revision 1.1.2.5  1997/10/24 03:39:35  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.4  1997/10/22 02:40:51  pvo
** Too many changes to list out here.
**
** Revision 1.1.2.3  1997/09/27 02:47:20  pvo
** Checked in.
**
***********************************************************************/
extern "C" {
#include <string.h>
}

#include "wizard.h"
#include "dialog.h"
//
// CLASS: Wizard
//
int
Wizard::start()
{
   if (_lastDialog < 0)
      return -1;

   return startDialog(0);
}

int
Wizard::cont()
{
   if (_lastDialog < 0)
      return -1;

   return startDialog(++_currentDialog);
}

int
Wizard::startDialog(int which)
{
   int rc = 1;
   DialogAction action;

   // Reset current Dialog
   if (which < 0 || _lastDialog < which)
   {
      rc = -1;
   }

   // Reset _currentDialog

   _currentDialog = which;
   if (_myDialogs[_currentDialog] == NULL)
   {
      rc = -1;
   }

   while (rc > 0)
   {
      Dialog *dl;

      dl = _myDialogs[_currentDialog];
      if (dl)
      {
         action = dl->execute();
      }
      else
      {
         rc = -1;
         break;
      }

      if (action == DIALOG_NEXT)
      {
         if (_currentDialog < _lastDialog)
         {
            _currentDialog = _currentDialog + 1;
         }
         else
         {
            rc = 0; // No more dialogs
         }
      }
      else if (action == DIALOG_PREV)
      {
         if (_currentDialog)
            _currentDialog--; 
         while (_currentDialog && _myDialogs[_currentDialog]->isHidden() == True)
            _currentDialog--;            
      }
      else if (action == DIALOG_EXIT)
      {
         rc = 0; 
      }
      else if (action == DIALOG_ERROR)
      {
         rc = -1;
      }
      else if (action == DIALOG_TERMINATE)
      {
         rc = -2;
      }
   }
   return rc;
}

void
Wizard::clear()
{
   // Relinquish all my dialogs
   _lastDialog = _currentDialog = -1;
   memset(_myDialogs, 0, sizeof(Dialog *) * MAX_DIALOGS);
}

void
Wizard::resetLast()
{
   // Reset all dialogs after the current dialog

   int i = _currentDialog + 1;
   while (i <= _lastDialog)
   {
      if (_myDialogs[i])
      {
        _myDialogs[i++]->unVisited();
      }
   }
   _lastDialog = _currentDialog;
}


void
Wizard::add(Dialog *p)
{
   _lastDialog = _currentDialog+1;
   _myDialogs[_lastDialog] = p;
}

void
Wizard::addLast(Dialog *p)
{
   _myDialogs[++_lastDialog] = p;
}
