#
# BEGIN COPYRIGHT BLOCK
# Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation version
# 2.1 of the License.
#  
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#  
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# END COPYRIGHT BLOCK
#
#
# These are the default values 
#
# Individual modules may override them
#
# BUILD_BOMB=[-DPUMPKIN_HOUR=xxxxxxx or just leave it empty]
# BUILD_DEBUG=[full, optimize, purify]
# BUILD_MODULE=[HTTP_ADMIN, HTTP_PERSONAL, HTTP_ENTERPRISE, ...]
# BUILD_SECURITY=[none, export, domestic]

ABS_ROOT:=$(shell cd $(BUILD_ROOT); pwd)
ABS_ROOT_PARENT := $(shell cd $(ABS_ROOT)/..; pwd)
RELTOOLSPATH = $(ABS_ROOT_PARENT)/reltools

TMP_ARCH := $(shell uname -s)
ifeq ($(TMP_ARCH), Windows_NT)
	TMP_ARCH=WINNT
endif
RELEASE := $(shell uname -r)
ifeq ($(TMP_ARCH), WINNT)
BUILD_ARCH = $(TMP_ARCH)
else
BUILD_ARCH := $(shell $(ABS_ROOT)/nsarch)
endif

PKG_ARCH=$(BUILD_ARCH)

PUMPKIN_AGE := 90
BUILD_BOMB=
BUILD_DEBUG=full
BUILD_MODULE=HTTP_ADMIN
BUILD_SECURITY=domestic
ifeq ($(USE_PTHREADS), 1)
  ifeq ($(USE_N32), 1)
     BUILD_PTHREADS=-n32-pth
  else
     BUILD_PTHREADS=-pth
  endif
else
  BUILD_PTHREADS=
endif

ifeq ($(USE_64), 1)
  NS64TAG = _64
else
  ifeq ($(BUILD_ARCH), HPUX)
    ifeq ($(NSOS_TEST1),ia64)
      NS64TAG=_32
    endif
  endif
endif

PRETTY_ARCH := $(shell uname -s)
 
NSOS_ARCH  := $(subst /,_,$(shell uname -s))
NSOS_TEST1 := $(shell uname -m)

ifeq ($(TMP_ARCH), SunOS)
  ifeq ($(NSOS_TEST1), i86pc)
	ARCHTYPE= _i86pc
  else
	ARCHTYPE=
  endif
else
  ARCHTYPE=
endif

ifeq ($(NSOS_ARCH),WINNT)
LIB_SUFFIX = lib
else
LIB_SUFFIX = a
endif
    
ifeq ($(NSOS_ARCH), IRIX64)
NSOS_ARCH := IRIX
endif
       
ifeq ($(NSOS_ARCH), AIX)
NSOS_TEST        := $(shell uname -v)
ifeq ($(NSOS_TEST),3)
NSOS_RELEASE   := $(shell uname -r)
else
NSOS_RELEASE   := $(shell uname -v).$(shell uname -r)
endif
else
NSOS_RELEASE      := $(shell uname -r)
#
# Handle FreeBSD 2.2-STABLE and Linux 2.0.30-osfmach3 and 2.2.14-5.0smp
#
ifeq (,$(filter-out Linux FreeBSD,$(NSOS_ARCH)))
  NSOS_RELEASE  := $(shell echo $(NSOS_RELEASE) | sed 's/-.*//')
endif
endif

# Catch NCR butchering of SVR4
ifeq ($(NSOS_ARCH),UNIX_SV)
ifneq ($(findstring NCR, $(shell grep NCR /etc/bcheckrc | head -1 )),)
NSOS_ARCH	:= NCR
PRETTY_ARCH     := NCR
else
# Make UnixWare something human readable
NSOS_ARCH	:= UNIXWARE
PRETTY_ARCH     := UNIXWARE
endif
# Get the OS release number, not 4.2
NSOS_RELEASE	:= $(shell uname -v)
endif

# Get OSF's OS revision number to figure out the OS release
ifeq ($(NSOS_ARCH),OSF1)
  NSOS_RELEASE1      := $(shell uname -r)
  NSOS_OSF1REV       := $(shell uname -v)
  ifeq ($(NSOS_OSF1REV),878)
    NSOS_RELEASE := $(NSOS_RELEASE1)D
  else
    ifeq ($(NSOS_OSF1REV),564)
      NSOS_RELEASE := $(NSOS_RELEASE1)B
    else
      ifeq ($(NSOS_OSF1REV),464)
        NSOS_RELEASE := $(NSOS_RELEASE1)A
      else
        NSOS_RELEASE := $(NSOS_RELEASE1)
      endif
    endif
  endif
endif

# Clean up SCO
ifeq ($(NSOS_ARCH),SCO_SV)
NSOS_ARCH	:= SCOOS
ifeq (5.0,$(findstring 5.0,$(shell ls /var/opt/K/SCO/Unix)))
NSOS_RELEASE	:= 5.0
else
NSOS_RELEASE	:= UNKNOWN
endif
endif

# Check if we're on RHEL
ifeq ($(NSOS_ARCH), Linux)
  NSOS_TEST := $(shell cat /etc/redhat-release)
  ifeq ($(findstring Taroon, $(NSOS_TEST)),Taroon)
    NSOS_ARCH = RHEL
    NSOS_RELEASE = 3
    # Always use gcc on RHEL
    GCC_VERSION = gcc$(word 1, $(shell gcc --version | sed 's/gcc.*GCC.\s//' | sed 's/\..*//'))
  else
    ifeq ($(findstring Nahant, $(NSOS_TEST)),Nahant)
      NSOS_ARCH = RHEL
      NSOS_RELEASE = 4
      # Always use gcc on RHEL
      GCC_VERSION = gcc$(word 1, $(shell gcc --version | sed 's/gcc.*GCC.\s//' | sed 's/\..*//'))
    else
# generic Linux
      NSOS_ARCH := Linux
# just keep major.minor release version e.g.
# 2.6.13-1.1532_FC4 -> 2.6
# also 10.11.4444 -> 10.11
	  NSOS_RELEASE := $(shell echo $(NSOS_RELEASE) | sed 's/\([0-9][0-9]*\.[0-9][0-9]*\).*/\1/')
    endif
  endif
endif

ifdef BUILD_INTL
INTL_DEF=-intl
endif

NSPLATFORM_DIR=$(NSOS_ARCH)$(NSOS_RELEASE)$(ARCHTYPE)$(NS64TAG)
PLATFORM_DEST=$(NSOBJDIR_NAME)
PLATFORM_OBJDIR=$(ABS_ROOT)/built/$(PLATFORM_DEST)
PLATFORM_PACKAGE_DIR=$(ABS_ROOT)/built/package/$(PLATFORM_DEST)
DEFAULT_OBJDIR=$(PLATFORM_OBJDIR)

ifneq ($(NO_BUILD_NUM), true)
GET_BUILD_NUM := $(shell cat $(ABS_ROOT)/$(NSPLATFORM_DIR)/buildnum.dat)
endif

ifdef REL_BUILD
INTGBUILD=0
else
INTGBUILD=1
endif

# Foreign language support
WEBSERVER_LANGS = ja fr de


ifeq ($(BUILD_ARCH), WINNT)
  BUILD_DLL_VERSION=yes
  BUILD_HTTPDLL_NAME=ns-httpd30
  BUILD_ADMIN_NAME=ns-admin40
  PERL_ARCH=$(shell uname -p)
  PERL=perl
  ZIPDIR=$(ABS_ROOT_PARENT)/components

  ZIP=$(ZIPDIR)/zip.exe
  UNZIP = $(ZIPDIR)/unzip.exe -o
else
  ifeq ($(BUILD_ARCH), Linux)
    PERL=perl
  else
    NSPERL_RELDATE :=20020626
    PERL=/share/builds/sbstools/nsPerl/$(NSPERL_RELDATE)/$(NSOBJDIR_NAME_32)/nsperl
  endif
  ifeq ($(TMP_ARCH), SunOS)
    NSOS_TEST1       := $(shell uname -m)
    ifeq ($(NSOS_TEST1), i86pc)
      ZIP = /bin/zip
      UNZIP = /bin/unzip -o
    else
      ifeq ($(INTERNAL_BUILD), 1)
        ZIP = $(COMPONENTS_DIR)/infozip/$(PKG_ARCH)/zip
        UNZIP = $(COMPONENTS_DIR)/infozip/$(PKG_ARCH)/unzip -o
      else
        ZIP = /usr/bin/zip
        UNZIP = /usr/bin/unzip -o
	  endif
    endif
  endif
  ifeq ($(INTERNAL_BUILD), 1)
    ZIP = $(COMPONENTS_DIR)/infozip/$(PKG_ARCH)/zip
    UNZIP = $(COMPONENTS_DIR)/infozip/$(PKG_ARCH)/unzip -o
  else
    ZIP = zip
    UNZIP = unzip -o
  endif
  IZIP  = $(ZIP) -ry9uT
endif


define echo_build_parms
echo ==== Building with the following parameters ====
echo BUILD_ARCH=$(BUILD_ARCH) 
echo BUILD_MODULE=$(BUILD_MODULE)
echo BUILD_SECURITY=$(BUILD_SECURITY)
echo BUILD_DEBUG=$(BUILD_DEBUG)
echo BUILD_BOMB=$(BUILD_BOMB)
echo BUILD_DLL_VERSION=$(BUILD_DLL_VERSION)
echo BUILD_HTTPDLL_NAME=$(BUILD_HTTPDLL_NAME)
echo BUILD_ADMIN_NAME=$(BUILD_ADMIN_NAME)
endef

#
# Set up these names because most of the makefiles use them now.
#
ARCH=$(BUILD_ARCH)
SECURITY=$(BUILD_SECURITY)
DEBUG=$(BUILD_DEBUG)
BOMB=$(BUILD_BOMB)
BUILD_DLL=$(BUILD_DLL_VERSION)
ADMINDLL_NAME=$(BUILD_ADMIN_NAME)
HTTPDLL_NAME=$(BUILD_HTTPDLL_NAME)
RESOLV_FLAG=
DO_SEARCH=
NODE_FLAG=-DNO_NODELOCK

MAJOR_VERSION:="1"
MINOR_VERSION:="0"
SETUP_VERSION:=1.0
LDAP_NO_LIBLCACHE:=1

DIRVERDIR=$(COMMON_OBJDIR)/include
DIRVER_H=$(DIRVERDIR)/dirver.h

#
# -- General Server Section ---------------------------------------------------
#

ifeq ($(BUILD_MODULE), SetupSDK)
PRODUCT="Fedora Setup SDK"
DIR=installer
endif

#
# -- Web Server Section -------------------------------------------------------
#

ifeq ($(BUILD_MODULE), HTTP_ADMIN)
PRODUCT="Fedora Administration Server"
DIR=admin
DO_CLUSTER=yes
BUILD_SNMP=yes
endif

ifeq ($(BUILD_MODULE), SETUP_ADMIN)
PRODUCT="Fedora Setup"
NS_PRODUCT=MCC_ADMSERV
endif

ifeq ($(BUILD_MODULE), SETUP_BASE)
PRODUCT="Fedora Servers Base Package"
DIR=base
endif

ifeq ($(BUILD_MODULE), SETUP_MCC)
PRODUCT="Fedora Mission Control package"
DIR=mcc
endif

ifeq ($(BUILD_MODULE), UNINST_SHELL)
PRODUCT="Fedora Uninstall Shell"
DIR=installer
endif

ifeq ($(BUILD_MODULE), UNINST_ADMIN)
PRODUCT="Fedora Administration Uninstaller"
DIR=admin
endif

#
# -- Default Section --------------------------------------------------------
#
# Some of the _OBJDIR is maintained for backward compatibility until they
# are all cleaned up.  Most of them heavily dependent on value of $(DIR)
#

ifndef ADMSERV_OBJDIR
ADMSERV_OBJDIR=$(DEFAULT_OBJDIR)
endif

ifndef COMMON_OBJDIR
COMMON_OBJDIR=$(DEFAULT_OBJDIR)
endif

ifndef OBJDIR
OBJDIR=$(DEFAULT_OBJDIR)
endif

